@@ -37,10 +37,10 @@ except ExpectedImportError:
     pyrex_seq_align_module = None
 
 __author__ = "Peter Maxwell"
-__copyright__ = "Copyright 2007-2011, The Cogent Project"
+__copyright__ = "Copyright 2007-2012, The Cogent Project"
 __credits__ = ["Peter Maxwell", "Gavin Huttley", "Rob Knight"]
 __license__ = "GPL"
-__version__ = "1.5.1"
+__version__ = "1.5.3"
 __maintainer__ = "Peter Maxwell"
 __email__ = "pm67nz@gmail.com"
 __status__ = "Production"
@@ -199,7 +199,7 @@ class Pair(object):
         self.alphabet = alignable1.alphabet
         
         for alignable in alignables:
-            assert isinstance(alignable, _Alignable), type(seq)
+            assert isinstance(alignable, _Alignable), type(alignable)
             if not isinstance(alignable, AlignableSeq):
                 some_pogs = True
                 break
