@@ -8,10 +8,10 @@ from cogent.core.info import Info
 from cogent.util import progress_display as UI
 
 __author__ = "Peter Maxwell"
-__copyright__ = "Copyright 2007-2011, The Cogent Project"
+__copyright__ = "Copyright 2007-2012, The Cogent Project"
 __credits__ = ["Peter Maxwell", "Gavin Huttley"]
 __license__ = "GPL"
-__version__ = "1.5.1"
+__version__ = "1.5.3"
 __maintainer__ = "Peter Maxwell"
 __email__ = "pm67nz@gmail.com"
 __status__ = "Production"
@@ -66,13 +66,14 @@ def TreeAlign(model, seqs, tree=None, indel_rate=0.01, indel_length=0.01,
                                     if param not in _exclude_params]
         else:
             est_params = None
+        
         dcalc = EstimateDistances(seqs, model, do_pair_align=True,
                                     est_params=est_params)
         dcalc.run()
         dists = dcalc.getPairwiseDistances()
         tree = NJ.nj(dists)
     
-    LF = model.makeLikelihoodFunction(tree.bifurcating(), aligned=False)
+    LF = model.makeLikelihoodFunction(tree.bifurcating(name_unnamed=True), aligned=False)
     if ests_from_pairwise and not param_vals:
         # we use the Median to avoid the influence of outlier pairs
         param_vals = {}
