@@ -1,18 +1,18 @@
 #!/usr/bin/env python
 
 from cogent.app.util import CommandLineApplication,\
-    CommandLineAppResult, ResultPath
+    CommandLineAppResult, ResultPath, ApplicationError
 from cogent.app.parameters import Parameter, FlagParameter, ValuedParameter,\
     MixedParameter,Parameters
-import commands
+
 from sys import platform
 from os  import remove,system,mkdir,getcwd,close
 
 __author__ = "Shandy Wikman"
-__copyright__ = "Copyright 2007-2009, The Cogent Project"
+__copyright__ = "Copyright 2007-2012, The Cogent Project"
 __contributors__ = ["Shandy Wikman", "Daniel McDonald"]
 __license__ = "GPL"
-__version__ = "1.5.0"
+__version__ = "1.5.3"
 __maintainer__ = "Shandy Wikman"
 __email__ = "ens01svn@cs.umu.se"
 __status__ = "Development"
@@ -207,8 +207,6 @@ class Dynalign(CommandLineApplication):
         if self._command is None:
             raise ApplicationError, '_command has not been set.'
         command = self._command
-        parameters = self.Parameters
-        synonyms = self._synonyms
         
         command_part1.append(cd_command)
         command_part1.append(command)
