@@ -5,9 +5,12 @@ mothur Version 1.6.0
 
 
 from __future__ import with_statement
-from os import path, getcwd, mkdir, remove, listdir
+from operator import attrgetter
+from os import path, getcwd, mkdir, remove, listdir, rmdir
+import re
 from shutil import copyfile
 from subprocess import Popen
+from tempfile import mkdtemp, NamedTemporaryFile
 from cogent.app.parameters import ValuedParameter
 from cogent.app.util import CommandLineApplication, ResultPath, \
     CommandLineAppResult, ApplicationError
@@ -15,10 +18,10 @@ from cogent.parse.mothur import parse_otu_list
 
 
 __author__ = "Kyle Bittinger"
-__copyright__ = "Copyright 2007-2011, The Cogent Project"
+__copyright__ = "Copyright 2007-2012, The Cogent Project"
 __credits__ = ["Kyle Bittinger"]
 __license__ = "GPL"
-__version__ = "1.5.1"
+__version__ = "1.5.3"
 __maintainer__ = "Kyle Bittinger"
 __email__ = "kylebittinger@gmail.com"
 __status__ = "Prototype"
@@ -198,27 +201,20 @@ class Mothur(CommandLineApplication):
         generated by Mothur.  It will raise an ApplicationError if no
         log file can be found.
 
-        Mothur 1.6.0 and 1.7.2 generate logfiles named
-        'mothur.logFile'.  Mothur 1.9.0 generates log files named in a
-        nondeterministic way, using the current time, with a lowercase
-        extension.  To work around this problem, we search the working
-        directory for files with the either extension.  If the working
-        directory contains two valid Mothur log files, this method may
-        inadvertently return the incorrect log file for Mothur >=
-        1.9.0.
+        Mothur generates log files named in a nondeterministic way,
+        using the current time.  We return the log file with the most
+        recent time, although this may lead to incorrect log file
+        detection if you are running many instances of mothur
+        simultaneously.
         """
-        input_dir = path.dirname(path.abspath(self._input_filename))
-        possible_logfiles = listdir(input_dir)
-        matching_logfiles = filter(
-            lambda x: x.endswith('.logfile') or x.endswith('.logFile'),
-            possible_logfiles)
-        try:
-            log_path = matching_logfiles[0]
-        except IndexError:
+        filenames = listdir(self.WorkingDir)
+        lognames = [x for x in filenames if re.match("^mothur\.\d+\.logfile$", x)]
+        if not lognames:
             raise ApplicationError(
                 'No log file detected in directory %s. Contents: \n\t%s' % (
                     input_dir, '\n\t'.join(possible_logfiles)))
-        return path.join(input_dir, log_path)
+        most_recent_logname = sorted(lognames, reverse=True)[0]
+        return path.join(self.WorkingDir, most_recent_logname)
 
     def _derive_unique_path(self):
         """Guess unique sequences path produced by Mothur"""
@@ -285,8 +281,7 @@ class Mothur(CommandLineApplication):
 
         data: a multiline string to be written to a file.
         """
-        self._input_filename = self.getTmpFilename(
-            self.WorkingDir, suffix='.fasta')
+        self._input_filename = self.getTmpFilename(suffix='.fasta')
         with open(self._input_filename, 'w') as f:
             f.write(data)
         return self._input_filename
@@ -301,8 +296,7 @@ class Mothur(CommandLineApplication):
             element before writing to a file in order to avoid
             multiple new lines accidentally be written to a file
         """
-        self._input_filename = self.getTmpFilename(
-            self.WorkingDir, suffix='.fasta')
+        self._input_filename = self.getTmpFilename(suffix='.fasta')
         with open(self._input_filename, 'w') as f:
             # Use lazy iteration instead of list comprehension to
             # prevent reading entire file into memory
@@ -316,8 +310,7 @@ class Mothur(CommandLineApplication):
 
         data: path or filename
         """
-        self._input_filename = self.getTmpFilename(
-            self.WorkingDir, suffix='.fasta')
+        self._input_filename = self.getTmpFilename(suffix='.fasta')
         copyfile(data, self._input_filename)
         return self._input_filename
 
@@ -346,6 +339,7 @@ class Mothur(CommandLineApplication):
 
     WorkingDir = property(_get_WorkingDir, _set_WorkingDir)
 
+
 def mothur_from_file(file):
     app = Mothur(InputHandler='_input_as_lines')
     result = app(file)
@@ -354,3 +348,125 @@ def mothur_from_file(file):
     result.cleanUp()
     return otus
 
+
+class MothurClassifySeqs(Mothur):
+    _options = {
+        'reference': ValuedParameter(
+            Name='reference', Value=None, Delimiter='=', Prefix=''),
+        'taxonomy': ValuedParameter(
+            Name='taxonomy', Value=None, Delimiter='=', Prefix=''),
+        'cutoff': ValuedParameter(
+            Name='cutoff', Value=None, Delimiter='=', Prefix=''),
+        'iters': ValuedParameter(
+            Name='iters', Value=None, Delimiter='=', Prefix=''),
+        'ksize': ValuedParameter(
+            Name='ksize', Value=None, Delimiter='=', Prefix=''),
+        }
+    _parameters = {}
+    _parameters.update(_options)
+
+    def _format_function_arguments(self, opts):
+        """Format a series of function arguments in a Mothur script."""
+        params = [self.Parameters[x] for x in opts]
+        return ', '.join(filter(None, map(str, params)))
+
+    def _compile_mothur_script(self):
+        """Returns a Mothur batch script as a string"""
+        fasta = self._input_filename
+
+        required_params = ["reference", "taxonomy"]
+        for p in required_params:
+            if self.Parameters[p].Value is None:
+                raise ValueError("Must provide value for parameter %s" % p)
+        optional_params = ["ksize", "cutoff", "iters"]
+        args = self._format_function_arguments(
+            required_params + optional_params)
+        script = '#classify.seqs(fasta=%s, %s)' % (fasta, args)
+        return script
+
+    def _get_result_paths(self):
+        input_base, ext = path.splitext(path.basename(self._input_filename))
+        result_by_suffix = {
+            ".summary": "summary",
+            ".taxonomy": "assignments",
+            ".accnos": "accnos",
+            }
+        
+        paths = {'log': self._derive_log_path()}
+        input_dir = path.dirname(self._input_filename)
+        for fn in listdir(input_dir):
+            if fn.startswith(input_base):
+                for suffix, result_key in result_by_suffix.items():
+                    if fn.endswith(suffix):
+                        paths[result_key] = path.join(input_dir, fn)
+        return dict([(k, ResultPath(v)) for (k,v) in paths.items()])
+
+
+def parse_mothur_assignments(lines):
+    for line in lines:
+        line = line.strip()
+        if not line:
+            continue
+        seq_id, _, assignment = line.partition("\t")
+        toks = assignment.rstrip(";").split(";")
+        lineage = []
+        conf = None
+        for tok in toks:
+            matchobj = re.match("(.+)\((\d+)\)$", tok)
+            if matchobj:
+                lineage.append(matchobj.group(1))
+                pct_conf = int(matchobj.group(2))
+                conf = pct_conf / 100.0
+        yield seq_id, lineage, conf
+
+
+def mothur_classify_file(
+    query_file, ref_fp, tax_fp, cutoff=None, iters=None, ksize=None,
+    output_fp=None):
+    """Classify a set of sequences using Mothur's naive bayes method
+
+    Dashes are used in Mothur to provide multiple filenames.  A
+    filepath with a dash typically breaks an otherwise valid command
+    in Mothur.  This wrapper script makes a copy of both files, ref_fp
+    and tax_fp, to ensure that the path has no dashes.
+
+    For convenience, we also ensure that each taxon list in the
+    id-to-taxonomy file ends with a semicolon.
+    """
+    tmp_ref_file = NamedTemporaryFile(suffix=".ref.fa")
+    for line in open(ref_fp):
+        tmp_ref_file.write(line)
+    tmp_ref_file.seek(0)
+
+    tmp_tax_file = NamedTemporaryFile(suffix=".tax.txt")
+    for line in open(tax_fp):
+        line = line.rstrip()
+        if not line.endswith(";"):
+            line = line + ";"
+        tmp_tax_file.write(line)
+        tmp_tax_file.write("\n")
+    tmp_tax_file.seek(0)
+
+    params = {"reference": tmp_ref_file.name, "taxonomy": tmp_tax_file.name}
+    if cutoff is not None:
+        params["cutoff"] = cutoff
+    if ksize is not None:
+        params["ksize"] = ksize
+    if iters is not None:
+        params["iters"] = iters
+
+    app = MothurClassifySeqs(params, InputHandler='_input_as_lines')
+    result = app(query_file)
+
+    # Force evaluation so we can safely clean up files
+    assignments = list(parse_mothur_assignments(result['assignments']))
+    result.cleanUp()
+
+    if output_fp is not None:
+        f = open(output_fp, "w")
+        for query_id, taxa, conf in assignments:
+            taxa_str = ";".join(taxa)
+            f.write("%s\t%s\t%.2f\n" % (query_id, taxa_str, conf))
+        f.close()
+        return None
+    return dict((a, (b, c)) for a, b, c in assignments)
