@@ -10,19 +10,26 @@ from cogent.core.alignment import SequenceCollection, Alignment
 from cogent.parse.tree import DndParser
 from cogent.core.tree import PhyloNode
 from cogent.parse.fasta import MinimalFastaParser
+from cogent.util.warning import deprecated
 
 __author__ = "Rob Knight"
-__copyright__ = "Copyright 2007-2011, The Cogent Project"
+__copyright__ = "Copyright 2007-2012, The Cogent Project"
 __credits__ = ["Micah Hamady", "Zongzhi Liu", "Mike Robeson",
-       "Catherine Lozupone", "Rob Knight", "Daniel McDonald", "Jeremy Widmann"]
+               "Catherine Lozupone", "Rob Knight", "Daniel McDonald",
+               "Jeremy Widmann", "Jai Ram Rideout"]
 __license__ = "GPL"
-__version__ = "1.5.1"
+__version__ = "1.5.3"
 __maintainer__ = "Micah Hamady"
 __email__ = "hamady@colorado.edu"
 __status__ = "Prototype"
 
 class Muscle(CommandLineApplication):
     """Muscle application controller"""
+
+    deprecated('class',
+               'cogent.app.muscle.Muscle',
+               'cogent.app.muscle_v38.Muscle',
+               '1.6')
     
     _options ={
         # Minimum spacing between anchor columns. [Integer]
@@ -78,6 +85,9 @@ class Muscle(CommandLineApplication):
         
         # Maximum number of iterations.
         '-maxiters':ValuedParameter('-',Name='maxiters',Delimiter=' '),
+
+        # Maximum memory in Mb
+        '-maxmb': ValuedParameter('-', Name='maxmb', Delimiter=' '),
         
         # Maximum number of new trees to build in iteration 2.
         '-maxtrees':ValuedParameter('-',Name='maxtrees',Delimiter=' '),
@@ -234,7 +244,6 @@ class Muscle(CommandLineApplication):
     
     _parameters = {}
     _parameters.update(_options)
-#    _command = "/Applications/muscle/muscle3.52_osx"
     _command = "muscle"
     
     def _input_as_seqs(self,data):
