@@ -3,10 +3,10 @@
 """
 
 __author__ = "Kyle Bittinger"
-__copyright__ = "Copyright 2007-2011, The Cogent Project"
+__copyright__ = "Copyright 2007-2012, The Cogent Project"
 __credits__ = ["Kyle Bittinger","Greg Caporaso"]
 __license__ = "GPL"
-__version__ = "1.5.1"
+__version__ = "1.5.3"
 __maintainer__ = "Kyle Bittinger"
 __email__ = "kylebittinger@gmail.com"
 __status__ = "Prototype"
@@ -200,259 +200,12 @@ class RdpClassifier(CommandLineApplication):
         return {'Assignments': ResultPath(assignment_fp, IsWritten=True)}
 
 
-class RdpClassifier20(RdpClassifier):
-    """RDP Classifier version 2.0 application controller
-
-    The RDP Classifier program is distributed as a java archive (.jar)
-    file.  If the file 'rdp_classifier-2.0.jar' is not found in the
-    current directory, the app controller looks in the directory
-    specified by the environment variable RDP_JAR_PATH.  If this
-    variable is not set, and 'rdp_classifier-2.0.jar' is not found in
-    the current directory, the application controller raises an
-    ApplicationNotFoundError.
-
-    The RDP Classifier often requires memory in excess of Java's
-    default 64M. To correct this situation, the authors recommend
-    increasing the maximum heap size for the java virtual machine.  An
-    option '-Xmx' (default 1000M) is provided for this purpose.
-    Details on this option may be found at
-    http://java.sun.com/j2se/1.5.0/docs/tooldocs/solaris/java.html
-
-    The classifier may optionally use a custom training set.  The full
-    path to the training set may be provided in the option
-    '-training-data'.
-    """
-    _input_handler = '_input_as_multiline_string'
-    _command = "rdp_classifier-2.0.jar"
-    _options ={}
-
-    # The following are available in the attributes JvmParameters,
-    # JarParameters, and PositionalParameters
-
-    _jvm_synonyms = {}
-    _jvm_parameters = {
-        # Maximum heap size for JVM.
-        '-Xmx': ValuedParameter('-', Name='Xmx', Delimiter='', Value='1000m'),
-        }
-    _positional_synonyms = {}
-    _positional_parameters = {
-        '-training-data': ValuedParameter('', Name='', Delimiter='', Value='', IsPath=True),
-        }
-
-    _parameters = {}
-    _parameters.update(_options)
-    _parameters.update(_jvm_parameters)
-    _parameters.update(_positional_parameters)
-
-    def getHelp(self):
-        """Returns documentation string"""
-        # Summary paragraph copied from rdp_classifier-2.0, which is
-        # licensed under the GPL 2.0 and Copyright 2008 Michigan State
-        # University Board of Trustees
-        help_str =\
-        """
-        Ribosomal Database Project - Classifier
-        http://rdp.cme.msu.edu/classifier/
-
-        The RDP Classifier is a naive Bayesian classifier which was
-        developed to provide rapid taxonomic placement based on rRNA
-        sequence data. The RDP Classifier can rapidly and accurately
-        classify bacterial 16s rRNA sequences into the new
-        higher-order taxonomy proposed by Bergey's Trust. It provides
-        taxonomic assignments from domain to genus, with confidence
-        estimates for each assignment. The RDP Classifier is not
-        limited to using the bacterial taxonomy proposed by the
-        Bergey's editors. It worked equally well when trained on the
-        NCBI taxonomy. The RDP Classifier likely can be adapted to
-        additional phylogenetically coherent bacterial taxonomies.
-
-        The following paper should be cited if this resource is used:
-
-        Wang, Q, G. M. Garrity, J. M. Tiedje, and J. R. Cole. 2007.
-        Naive Bayesian Classifier for Rapid Assignment of rRNA
-        Sequences into the New Bacterial Taxonomy.  Appl Environ
-        Microbiol. 73(16):5261-7.
-        """
-        return help_str
-
-    def __call__(self, data=None, remove_tmp=True):
-        """Run the application with the specified kwargs on data
-        
-            data: anything that can be cast into a string or written out to
-                a file. Usually either a list of things or a single string or 
-                number. input_handler will be called on this data before it 
-                is passed as part of the command-line argument, so by creating
-                your own input handlers you can customize what kind of data
-                you want your application to accept
-
-            remove_tmp: if True, removes tmp files
-        """
-        input_handler = self.InputHandler
-        suppress_stdout = self.SuppressStdout
-        suppress_stderr = self.SuppressStderr
-        assignment_fp = FilePath(self.getTmpFilename(self.TmpDir))
-        if suppress_stdout:
-            outfile = FilePath('/dev/null')
-        else:
-            outfile = FilePath(self.getTmpFilename(self.TmpDir))
-        if suppress_stderr:
-            errfile = FilePath('/dev/null')
-        else:
-            errfile = FilePath(self.getTmpFilename(self.TmpDir))
-        if data is None:
-            input_arg = ''
-        else:
-            input_arg = getattr(self,input_handler)(data)
-
-        training_data = self.PositionalParameters['-training-data']
-
-        # Build up the command, consisting of a BaseCommand followed by
-        # input and output (file) specifications
-        command = self._commandline_join(
-            [self.BaseCommand, input_arg, assignment_fp, training_data, 
-             '>', outfile, '2>', errfile,]
-            )
-
-        if self.HaltExec: 
-            raise AssertionError, "Halted exec with command:\n" + command
-        # The return value of system is a 16-bit number containing the signal 
-        # number that killed the process, and then the exit status. 
-        # We only want to keep the exit status so do a right bitwise shift to 
-        # get rid of the signal number byte
-        exit_status = system(command) >> 8
-      
-        # Determine if error should be raised due to exit status of 
-        # appliciation
-        if not self._accept_exit_status(exit_status):
-            raise ApplicationError, \
-             'Unacceptable application exit status: %s, command: %s'\
-                % (str(exit_status),command)
-        
-        # open the stdout and stderr if not being suppressed
-        out = None
-        if not suppress_stdout:
-            out = open(outfile,"r")
-        err = None        
-        if not suppress_stderr:
-            err = open(errfile,"r")
-
-        result_paths = self._get_result_paths(data)
-        result_paths['Assignments'] = ResultPath(assignment_fp)
-        result = CommandLineAppResult(
-            out, err, exit_status, result_paths=result_paths)
-
-        # Clean up the input file if one was created
-        if remove_tmp:
-            if self._input_filename:
-                remove(self._input_filename)
-                self._input_filename = None
-
-        return result
-
-    def _accept_exit_status(self, status):
-        """Returns false if an error occurred in execution
-        """
-        return (status == 0)
-
-    def _error_on_missing_application(self,params):
-        """Raise an ApplicationNotFoundError if the app is not accessible
-        """
-        command = self._get_jar_fp()
-        if not exists(command):
-            raise ApplicationNotFoundError,\
-             "Cannot find jar file. Is it installed? Is $RDP_JAR_PATH"+\
-             " set correctly?"
-
-    def _get_jar_fp(self):
-        """Returns the full path to the JAR file.
-
-        Raises an ApplicationError if the JAR file cannot be
-        found in the (1) current directory or (2) the path specified
-        in the RDP_JAR_PATH environment variable.
-        """
-        # handles case where the jar file is in the current working directory
-        if exists(self._command):
-            return self._command
-        # handles the case where the user has specified the location via
-        # an environment variable
-        elif 'RDP_JAR_PATH' in environ:
-            return getenv('RDP_JAR_PATH')
-        # error otherwise
-        else:
-            raise ApplicationError,\
-             "$RDP_JAR_PATH is not set -- this must be set to use the"+\
-             " RDP classifier application controller."
-
-    # Overridden to pull out JVM-specific command-line arguments.
-    def _get_base_command(self):
-        """Returns the base command plus command-line options.
-
-        Does not include input file, output file, and training set.
-        """
-        # Necessary? Preserve for consistency.
-        if self._command is None:
-            raise ApplicationError, '_command has not been set.'
-
-        # Append a change directory to the beginning of the command to change 
-        # to self.WorkingDir before running the command
-        # WorkingDir should be in quotes -- filenames might contain spaces
-        cd_command = ''.join(['cd ',str(self.WorkingDir),';'])
-
-        jvm_command = "java"
-        jvm_arguments = self._commandline_join(self.JvmParameters.values())
-        jar_arguments = '-jar "%s"' % self._get_jar_fp()
-
-        result = self._commandline_join(
-            [cd_command, jvm_command, jvm_arguments, jar_arguments]
-            )
-        return result
-    
-    BaseCommand = property(_get_base_command)
-
-    def _commandline_join(self, tokens):
-        """Formats a list of tokens as a shell command
-
-        This seems to be a repeated pattern; may be useful in
-        superclass.
-        """
-        commands = filter(None, map(str, tokens))
-        return self._command_delimiter.join(commands).strip()
-
-    @property
-    def JvmParameters(self):
-        return self.__extract_parameters('jvm')
-
-    @property
-    def PositionalParameters(self):
-        return self.__extract_parameters('positional')
-
-    def __extract_parameters(self, name):
-        """Extracts parameters in self._<name>_parameters from self.Parameters
-
-        Allows the program to conveniently access a subset of user-
-        adjusted parameters, which are stored in the Parameters
-        attribute.
-        
-        Relies on the convention of providing dicts named according to
-        "_<name>_parameters" and "_<name>_synonyms".  The main
-        parameters object is expected to be initialized with the
-        contents of these dicts.  This method will throw an exception
-        if either convention is not adhered to.
-        """
-        parameters = getattr(self, '_' + name + '_parameters')
-        synonyms   = getattr(self, '_' + name + '_synonyms')
-        result = Parameters(parameters, synonyms)
-        for key in result.keys():
-            result[key] = self.Parameters[key]
-        return result
-
-
 class RdpTrainer(RdpClassifier):
     _input_handler = '_input_as_lines'
     TrainingClass = 'edu.msu.cme.rdp.classifier.train.ClassifierTraineeMaker'
     PropertiesFile = 'RdpClassifier.properties'
 
-    _rdp_parameters = {
+    _parameters = {
         'taxonomy_file': ValuedParameter(None, None, IsPath=True),
         'model_output_dir': ValuedParameter(None, None, IsPath=True),
         'training_set_id': ValuedParameter(None, None, Value='1'),
@@ -463,14 +216,14 @@ class RdpTrainer(RdpClassifier):
         # Maximum heap size for JVM.
         '-Xmx': ValuedParameter('-', Name='Xmx', Delimiter='', Value='1000m'),
         }
-    _parameters = {}
-    _parameters.update(_rdp_parameters)
     _parameters.update(_jvm_parameters)
 
     def _get_base_command(self):
         """Returns the base command plus command-line options.
 
-        Does not include input file, output file, and training set.
+        Handles everything up to and including the classpath.  The
+        positional training parameters are added by the
+        _input_handler_decorator method.
         """
         cd_command = ''.join(['cd ', str(self.WorkingDir), ';'])
         jvm_command = "java"
@@ -489,16 +242,22 @@ class RdpTrainer(RdpClassifier):
         self.__InputHandler = method_name
 
     def _get_input_handler(self):
+        """Returns decorator that wraps the requested input handler.
+        """
         return '_input_handler_decorator'
 
     InputHandler = property(_get_input_handler, _set_input_handler)
 
     @property
     def ModelDir(self):
-        return os.path.abspath(self.Parameters['model_output_dir'])
+        """Absolute FilePath to the training output directory.
+        """
+        model_dir = self.Parameters['model_output_dir'].Value
+        absolute_model_dir = os.path.abspath(model_dir)
+        return FilePath(absolute_model_dir)
 
     def _input_handler_decorator(self, data):
-        """Appends trailing parameters to selected input_handler's results.
+        """Adds positional parameters to selected input_handler's results.
         """
         input_handler = getattr(self, self.__InputHandler)
         input_parts = [
@@ -512,21 +271,21 @@ class RdpTrainer(RdpClassifier):
         return self._commandline_join(input_parts)
 
     def _get_result_paths(self, output_dir):
+        """Return a dict of output files.
+        """
+        # Only include the properties file here. Add the other result
+        # paths in the __call__ method, so we can catch errors if an
+        # output file is not written.
         self._write_properties_file()
-        training_files = {
-            'bergeyTree': 'bergeyTrainingTree.xml',
-            'probabilityList': 'genus_wordConditionalProbList.txt',
-            'probabilityIndex': 'wordConditionalProbIndexArr.txt',
-            'wordPrior': 'logWordPrior.txt',
-            'properties': self.PropertiesFile,
-        }
-        result_paths = {}
-        for key, filename in training_files.iteritems():
-            result_paths[key] = ResultPath(
-                Path=os.path.join(self.ModelDir, filename), IsWritten=True)
+        properties_fp = os.path.join(self.ModelDir, self.PropertiesFile)
+        result_paths = {
+            'properties': ResultPath(properties_fp, IsWritten=True,)
+            }
         return result_paths
     
     def _write_properties_file(self):
+        """Write an RDP training properties file manually.
+        """
         # The properties file specifies the names of the files in the
         # training directory.  We use the example properties file
         # directly from the rdp_classifier distribution, which lists
@@ -545,7 +304,44 @@ class RdpTrainer(RdpClassifier):
             "November 2003\n"
             )
         properties_file.close()
+      
+    def __call__(self, data=None, remove_tmp=True):
+        """Run the application with the specified kwargs on data
 
+        data: anything that can be cast into a string or written out
+          to a file. Usually either a list of things or a single
+          string or number. input_handler will be called on this data
+          before it is passed as part of the command-line argument, so
+          by creating your own input handlers you can customize what
+          kind of data you want your application to accept
+
+        remove_tmp: if True, removes tmp files
+        """
+        result = super(RdpClassifier, self).__call__(data=data, remove_tmp=remove_tmp)
+        training_files = {
+            'bergeyTree': 'bergeyTrainingTree.xml',
+            'probabilityList': 'genus_wordConditionalProbList.txt',
+            'probabilityIndex': 'wordConditionalProbIndexArr.txt',
+            'wordPrior': 'logWordPrior.txt',
+        }
+        for key, training_fn in sorted(training_files.items()):
+            training_fp = os.path.join(self.ModelDir, training_fn)
+            if not os.path.exists(training_fp):
+                exception_msg = (
+                    "Training output file %s not found.  This may "
+                    "happen if an error occurred during the RDP training "
+                    "process.  More details may be available in the "
+                    "standard error, printed below.\n\n" % training_fp
+                    )
+                stderr_msg = result["StdErr"].read()
+                result["StdErr"].seek(0)
+                raise ApplicationError(exception_msg + stderr_msg)
+            # Not in try/except clause because we already know the
+            # file exists. Failure would be truly exceptional, and we
+            # want to maintain the original exception in that case.
+            result[key] = open(training_fp)
+        return result
+        
 
 def parse_command_line_parameters(argv=None):
     """ Parses command line arguments """
@@ -573,7 +369,7 @@ def parse_command_line_parameters(argv=None):
 
 def assign_taxonomy(
     data, min_confidence=0.80, output_fp=None, training_data_fp=None,
-    fixrank=True):
+    fixrank=True, max_memory=None):
     """Assign taxonomy to each sequence in data with the RDP classifier
     
         data: open fasta file object or list of fasta lines
@@ -592,14 +388,19 @@ def assign_taxonomy(
         seq_id_lookup[seq_id.split()[0]] = seq_id
     
     app = RdpClassifier()
-
+    if max_memory is not None:
+        app.Parameters['-Xmx'].on(max_memory)
+    
     temp_output_file = tempfile.NamedTemporaryFile(
         prefix='RdpAssignments_', suffix='.txt')
     app.Parameters['-o'].on(temp_output_file.name)
     if training_data_fp is not None:
         app.Parameters['-t'].on(training_data_fp)
+
     if fixrank:
         app.Parameters['-f'].on('fixrank')
+    else:
+        app.Parameters['-f'].on('allrank')
 
     app_result = app(data)
 
@@ -616,6 +417,8 @@ def assign_taxonomy(
     
     for line in app_result['Assignments']:
         rdp_id, direction, taxa = parse_rdp_assignment(line)
+        if taxa[0][0] == "Root":
+            taxa = taxa[1:]
         orig_id = seq_id_lookup[rdp_id]
         lineage, confidence = get_rdp_lineage(taxa, min_confidence)
         if lineage:
@@ -638,7 +441,8 @@ def assign_taxonomy(
         return assignments
 
 
-def train_rdp_classifier(training_seqs_file, taxonomy_file, model_output_dir):
+def train_rdp_classifier(
+    training_seqs_file, taxonomy_file, model_output_dir, max_memory=None):
     """ Train RDP Classifier, saving to model_output_dir
 
         training_seqs_file, taxonomy_file: file-like objects used to
@@ -652,20 +456,22 @@ def train_rdp_classifier(training_seqs_file, taxonomy_file, model_output_dir):
     Once the model data has been generated, the RDP Classifier may 
     """
     app = RdpTrainer()
+    if max_memory is not None:
+        app.Parameters['-Xmx'].on(max_memory)
     
     temp_taxonomy_file = tempfile.NamedTemporaryFile(
         prefix='RdpTaxonomy_', suffix='.txt')
     temp_taxonomy_file.write(taxonomy_file.read())
     temp_taxonomy_file.seek(0)
 
-    app.Parameters['taxonomy_file'] = temp_taxonomy_file.name
-    app.Parameters['model_output_dir'] = model_output_dir
+    app.Parameters['taxonomy_file'].on(temp_taxonomy_file.name)
+    app.Parameters['model_output_dir'].on(model_output_dir)
     return app(training_seqs_file)
-
+    
 
 def train_rdp_classifier_and_assign_taxonomy(
     training_seqs_file, taxonomy_file, seqs_to_classify, min_confidence=0.80, 
-    model_output_dir=None, classification_output_fp=None):
+    model_output_dir=None, classification_output_fp=None, max_memory=None):
     """ Train RDP Classifier and assign taxonomy in one fell swoop
 
     The file objects training_seqs_file and taxonomy_file are used to
@@ -688,11 +494,13 @@ def train_rdp_classifier_and_assign_taxonomy(
         training_dir = model_output_dir
 
     training_results = train_rdp_classifier(
-        training_seqs_file, taxonomy_file, training_dir)
+        training_seqs_file, taxonomy_file, training_dir, max_memory=max_memory)
     training_data_fp = training_results['properties'].name
+
     assignment_results = assign_taxonomy(
         seqs_to_classify, min_confidence=min_confidence, 
-        output_fp=classification_output_fp, training_data_fp=training_data_fp)
+        output_fp=classification_output_fp, training_data_fp=training_data_fp,
+        max_memory=max_memory, fixrank=False)
 
     if model_output_dir is None:
         rmtree(training_dir)
