@@ -1,5 +1,4 @@
 #!/usr/bin/env python
-import commands
 from sys import platform
 from os import remove,system,mkdir,getcwd,close,sep
 from random import choice
@@ -12,11 +11,11 @@ from cogent.util.transform import cartesian_product
 from cogent.util.misc import app_path
 
 __author__ = "Sandra Smit and Greg Caporaso"
-__copyright__ = "Copyright 2007-2011, The Cogent Project"
+__copyright__ = "Copyright 2007-2012, The Cogent Project"
 __credits__ = ["Greg Caporaso", "Sandra Smit", "Micah Hamady",
                     "Jeremy Widmann", "Rob Knight", "Daniel McDonald"]
 __license__ = "GPL"
-__version__ = "1.5.1"
+__version__ = "1.5.3"
 __maintainer__ = "Sandra Smit"
 __email__ = "sandra.smit@colorado.edu"
 __status__ = "Development"
@@ -246,8 +245,10 @@ class CommandLineApplication(Application):
         # appliciation
         if not self._accept_exit_status(exit_status):
             raise ApplicationError, \
-             'Unacceptable application exit status: %s, command: %s'\
-                % (str(exit_status),command)
+             'Unacceptable application exit status: %s\n' % str(exit_status) +\
+             'Command:\n%s\nStdOut:\n%s\nStdErr:\n%s\n' % (command, 
+                                                           open(outfile).read(), 
+                                                           open(errfile).read())
         # bash returns 127 as the exit status if the command could not
         # be found -- raise an ApplicationError on status == 127.
         # elif exit_status == 127:
@@ -264,9 +265,13 @@ class CommandLineApplication(Application):
         err = None        
         if not suppress_stderr:
             err = open(errfile,"r")
-       
-        result =  CommandLineAppResult(out,err,exit_status,\
-            result_paths=self._get_result_paths(data)) 
+            
+        try:
+            result = CommandLineAppResult(\
+             out,err,exit_status,result_paths=self._get_result_paths(data))
+        except ApplicationError:
+            result = self._handle_app_result_build_failure(\
+             out,err,exit_status,self._get_result_paths(data))
 
         # Clean up the input file if one was created
         if remove_tmp:
@@ -276,6 +281,16 @@ class CommandLineApplication(Application):
 
         return result
    
+    def _handle_app_result_build_failure(self,out,err,exit_status,result_paths):
+        """ Called when an ApplicationError is raised on building the CommandLineAppResult 
+        
+            This is useful for checking log files or other special handling in cases
+             when expected files aren't present.
+        
+        """
+        raise ApplicationError, "Error constructing CommandLineAppResult."
+   
+   
     def _input_as_string(self,data):
         """ Return data as a string """
         return str(data)
@@ -363,7 +378,6 @@ class CommandLineApplication(Application):
             raise ApplicationError, '_command has not been set.'
         command = self._command
         parameters = self.Parameters
-        synonyms = self._synonyms
         
         command_parts.append(cd_command)
         command_parts.append(command)
