@@ -46,11 +46,11 @@ from bisect import bisect_right, bisect_left
 import copy
 
 __author__ = "Rob Knight"
-__copyright__ = "Copyright 2007-2009, The Cogent Project"
+__copyright__ = "Copyright 2007-2012, The Cogent Project"
 __credits__ = ["Rob Knight", "Peter Maxwell", "Matthew Wakefield",
                     "Gavin Huttley"]
 __license__ = "GPL"
-__version__ = "1.5.0"
+__version__ = "1.5.3"
 __maintainer__ = "Rob Knight"
 __email__ = "rob@spot.colorado.edu"
 __status__ = "Prototype"
@@ -291,16 +291,17 @@ class Span(SpanI):
         # Find the right span(s) of the map
         first = bisect_right(offsets, zlo) - 1
         last = bisect_left(offsets, zhi, first) -1
-        result= spans[first:last+1]
+        result = spans[first:last+1]
         
         # Cut off something at either end to get
         # the same position and length as 'self'
         if result:
-            end_of_last = offsets[last]+spans[last].length
-            if end_of_last > zhi:
-                result[-1] = result[-1][:result[-1].length-(end_of_last-zhi)]
-            if zlo > offsets[first]:
-                result[0] = result[0][zlo-offsets[first]:]
+            end_trim = offsets[last] + spans[last].length - zhi
+            start_trim = zlo - offsets[first]
+            if end_trim > 0:
+                result[-1] = result[-1][:result[-1].length-end_trim]
+            if start_trim > 0:
+                result[0] = result[0][start_trim:]
         
         # May need to add a bit at either end if the span didn't lie entirely
         # within its parent map (eg: Display slice, inverse of feature map).
@@ -549,13 +550,18 @@ class Map(object):
             new_parts.append(span / scale)
         return Map(spans=new_parts, parent_length=self.parent_length // scale)
     
+    def __add__(self, other):
+        if other.parent_length != self.parent_length:
+            raise ValueError("Those maps belong to different sequences")
+        return Map(spans=self.spans + other.spans, parent_length=self.parent_length)
+        
     def withTerminiUnknown(self):
         return Map(self, spans=self.spans[:],
                 parent_length=self.parent_length,
                 termini_unknown = True)
     
     def getCoveringSpan(self):
-        if self.Reverse == -1:
+        if self.Reverse:
             span = (self.End, self.Start)
         else:
             span = (self.Start, self.End)
@@ -592,13 +598,13 @@ class Map(object):
         return Map(result, parent_length=self.parent_length)
     
     def reversed(self):
+        """Reversed location on same parent"""
         spans = [s.reversed() for s in self.spans]
         spans.reverse()
         return Map(spans=spans, parent_length=self.parent_length)
     
     def nucleicReversed(self):
-        """reverses spans according to concept of nucleic acid sequence
-        reverse complement"""
+        """Same location on reversed parent"""
         spans = [s.reversedRelativeTo(self.parent_length) for s in self.spans]
         return Map(spans=spans, parent_length=self.parent_length)
     
