@@ -30,11 +30,11 @@ import re
 import warnings
 
 __author__ = "Rob Knight, Gavin Huttley, and Peter Maxwell"
-__copyright__ = "Copyright 2007-2011, The Cogent Project"
+__copyright__ = "Copyright 2007-2012, The Cogent Project"
 __credits__ = ["Rob Knight", "Peter Maxwell", "Gavin Huttley",
                     "Matthew Wakefield", "Daniel McDonald"]
 __license__ = "GPL"
-__version__ = "1.5.1"
+__version__ = "1.5.3"
 __maintainer__ = "Rob Knight"
 __email__ = "rob@spot.colorado.edu"
 __status__ = "Production"
@@ -607,9 +607,8 @@ class Sequence(_Annotatable, SequenceI):
             else:
                 seg = self._seq[span.Start:span.End]
                 if span.Reverse:
-                    basepairs = self.MolType.Complements
-                    assert basepairs, 'unreversable'
-                    seg = [basepairs[base] for base in seg[::-1]]
+                    complement = self.MolType.complement
+                    seg = [complement(base) for base in seg[::-1]]
                     seg = ''.join(seg)
             yield seg
     
@@ -630,8 +629,6 @@ class Sequence(_Annotatable, SequenceI):
         # Called by generic __getitem__
         segments = self.gappedByMapSegmentIter(map, allow_gaps=False)
         new = self.__class__(''.join(segments), self.Name, Info=self.Info)
-        annots = self._slicedAnnotations(new, map)
-        new.annotations = annots
         return new
     
     def __add__(self, other):
@@ -737,7 +734,9 @@ class Sequence(_Annotatable, SequenceI):
         seq = self.__class__(
             ''.join(gapless),
             Name = self.getName(), Info=self.Info)
-        return map, seq
+        if self.annotations:
+            seq.annotations = [a.remappedTo(seq, map) for a in self.annotations]
+        return (map, seq)
     
 
 class ProteinSequence(Sequence):
