@@ -25,23 +25,23 @@ Definition of relevant terms or abbreviations:
        from a node
     -  stem: the edge immediately preceeding a clade
 """
-from numpy import zeros, argsort
+from numpy import zeros, argsort, ceil, log
 from copy import deepcopy
 import re
 from cogent.util.transform import comb
 from cogent.maths.stats.test import correlation
 from operator import or_
 from cogent.util.misc import InverseDict
-from random import shuffle
+from random import shuffle, choice
 
 __author__ = "Gavin Huttley, Peter Maxwell and Rob Knight"
-__copyright__ = "Copyright 2007-2011, The Cogent Project"
+__copyright__ = "Copyright 2007-2012, The Cogent Project"
 __credits__ = ["Gavin Huttley", "Peter Maxwell", "Rob Knight",
                     "Andrew Butterfield", "Catherine Lozupone", "Micah Hamady",
                     "Jeremy Widmann", "Zongzhi Liu", "Daniel McDonald",
                     "Justin Kuczynski"]
 __license__ = "GPL"
-__version__ = "1.5.1"
+__version__ = "1.5.3"
 __maintainer__ = "Gavin Huttley"
 __email__ = "gavin.huttley@anu.edu.au"
 __status__ = "Production"
@@ -593,6 +593,12 @@ class TreeNode(object):
         if len(tips) == 0:
             return None
 
+        # scrub tree
+        if hasattr(self, 'black'):
+            for n in self.traverse(include_self=True):
+                if hasattr(n, 'black'):
+                    delattr(n, 'black')
+
         for t in tips:
             prev = t
             curr = t.Parent
@@ -1401,6 +1407,58 @@ class TreeNode(object):
             if n.Name in mapping:
                 n.Name = mapping[n.Name]
 
+    def multifurcating(self, num, eps=None, constructor=None, \
+            name_unnamed=False):
+        """Return a new tree with every node having num or few children
+
+        num : the number of children a node can have max
+        eps : default branch length to set if self or constructor is of
+            PhyloNode type
+        constructor : a TreeNode or subclass constructor. If None, uses self
+        """
+        if num < 2:
+            raise TreeError, "Minimum number of children must be >= 2"
+
+        if eps is None:
+            eps = 0.0
+
+        if constructor is None:
+            constructor = self.__class__
+
+        if hasattr(constructor, 'Length'):
+            set_branchlength = True
+        else:
+            set_branchlength = False
+
+        new_tree = self.copy()
+
+        for n in new_tree.preorder(include_self=True):
+            while len(n.Children) > num:
+                new_node = constructor(Children=n.Children[-num:])
+
+                if set_branchlength:
+                    new_node.Length = eps
+
+                n.append(new_node)
+
+        if name_unnamed:
+            alpha = 'abcdefghijklmnopqrstuvwxyz'
+            alpha += alpha.upper()
+            base = 'AUTOGENERATED_NAME_%s'
+
+            # scale the random names by tree size
+            s = int(ceil(log(len(new_tree.tips())))) 
+
+            for n in new_tree.nontips():
+                if n.Name is None:
+                    n.Name = base % ''.join([choice(alpha) for i in range(s)])
+
+        return new_tree
+
+    def bifurcating(self, eps=None, constructor=None, name_unnamed=False):
+        """Wrap multifurcating with a num of 2"""
+        return self.multifurcating(2, eps, constructor, name_unnamed)
+
     def getNodesDict(self):
         """Returns a dict keyed by node name, value is node
 
@@ -1689,17 +1747,6 @@ class PhyloNode(TreeNode):
             result.Name = "root"
         return result
     
-    def bifurcating(self, constructor=None):
-        # With every node having 2 or fewer children.
-        if constructor is None:
-            constructor = self._default_tree_constructor()
-        children = [child.bifurcating(constructor) for child in self.Children]
-        while len(children) > 2:
-            extra = constructor(None, tuple(children[-2:]))
-            children[-2:] = [extra]
-        result = constructor(self, tuple(children))
-        return result
-    
     def balanced(self):
         """Tree 'rooted' here with no neighbour having > 50% of the edges.
         
