@@ -1,3 +1,4 @@
+import re
 import sqlalchemy as sql
 
 from cogent.db.ensembl.species import Species as _Species
@@ -14,10 +15,10 @@ from cogent.util.misc import flatten
 
 
 __author__ = "Gavin Huttley"
-__copyright__ = "Copyright 2007-2011, The Cogent Project"
+__copyright__ = "Copyright 2007-2012, The Cogent Project"
 __credits__ = ["Gavin Huttley"]
 __license__ = "GPL"
-__version__ = "1.5.1"
+__version__ = "1.5.3"
 __maintainer__ = "Gavin Huttley"
 __email__ = "Gavin.Huttley@anu.edu.au"
 __status__ = "alpha"
@@ -79,6 +80,8 @@ class Genome(object):
         if Release is None:
             Release = get_latest_release(account=account)
         
+        self._gen_release = None
+        
         # TODO make name and release immutable properties
         self.Species = _Species.getSpeciesName(Species)
         self.Release = str(Release)
@@ -106,6 +109,9 @@ class Genome(object):
                         species=self.Species, pool_recycle=self._pool_recycle)
         if self._core_db is None and db_type == 'core':
             self._core_db = Database(db_type='core', **connection)
+            gen_rel = self.CoreDb.db_name.GeneralRelease
+            gen_rel = int(re.findall(r'^\d+', str(gen_rel))[0])
+            self._gen_release = gen_rel
         elif self._var_db is None and db_type == 'variation':
             self._var_db = Database(db_type='variation', **connection)
         elif self._other_db is None and db_type == 'otherfeatures':
@@ -129,6 +135,15 @@ class Genome(object):
     
     OtherFeaturesDb = property(_get_other_db)
     
+    @property
+    def GeneralRelease(self):
+        """returns True if the general Ensembl release is >= 65"""
+        # General release is used here as to support Ensembl genomes
+        if self._gen_release is None:
+            self.CoreDb
+        
+        return self._gen_release
+    
     def _get_biotype_description_condition(self, gene_table, Description=None, BioType=None, like=True):
         assert Description or BioType, "no valid argument provided"
         btype, descr = None, None
@@ -155,8 +170,13 @@ class Genome(object):
         return condition
     
     def _build_gene_query(self, db, condition, gene_table, gene_id_table, xref_table=None):
-        join_obj = gene_id_table.join(gene_table, gene_id_table.c.gene_id==gene_table.c.gene_id)
-        select_obj = [gene_id_table.c.stable_id, gene_table]
+        if gene_id_table is None: # Ensembl releases later than >= 65
+            join_obj = gene_table
+            select_obj = [gene_table]
+        else:
+            join_obj = gene_id_table.join(gene_table, gene_id_table.c.gene_id==gene_table.c.gene_id)
+            select_obj = [gene_id_table.c.stable_id, gene_table]
+        
         if db.Type == 'core':
             join_obj = join_obj.outerjoin(xref_table, gene_table.c.display_xref_id==xref_table.c.xref_id)
             select_obj.append(xref_table.c.display_label)
@@ -186,11 +206,20 @@ class Genome(object):
                          BioType=None, synonym=None, like=True):
         xref_table = [None, db.getTable('xref')][db.Type == 'core']
         gene_table = db.getTable('gene')
-        gene_id_table = db.getTable('gene_stable_id')
+        
+        # after release 65, the gene_id_table is removed. The following is to maintain
+        # support for earlier releases
+        release_ge_65 = self.GeneralRelease >= 65
+        if release_ge_65:
+            gene_id_table = None
+        else:
+            gene_id_table = db.getTable('gene_stable_id')
         
         assert Symbol or Description or StableId or BioType, "no valid argument provided"
         if Symbol:
             condition = xref_table.c.display_label==Symbol
+        elif StableId and release_ge_65:
+            condition = gene_table.c.stable_id==StableId
         elif StableId:
             condition = gene_id_table.c.stable_id==StableId
         else:
@@ -332,13 +361,20 @@ class Genome(object):
         """returns all genes"""
         xref_table = [None, db.getTable('xref')][db.Type == 'core']
         gene_table = db.getTable('gene')
-        gene_id_table = db.getTable('gene_stable_id')
+        
+        # after release 65, the gene_id_table is removed. The following is to maintain
+        # support for earlier releases.
+        if self.GeneralRelease >= 65:
+            gene_id_table = None
+        else:
+            gene_id_table = db.getTable('gene_stable_id')
         
         # note gene records are at chromosome, not contig, level
         condition = gene_table.c.seq_region_id == query_coord.seq_region_id
         query = self._build_gene_query(db, condition, gene_table, gene_id_table, xref_table)
         query = location_query(gene_table, query_coord.EnsemblStart,
                     query_coord.EnsemblEnd, query=query, where=where_feature)
+        
         for record in query.execute():
             new = Coordinate(self, CoordName=query_coord.CoordName,
                             Start=record['seq_region_start'],
@@ -461,7 +497,13 @@ class Genome(object):
             query = var_feature_table.c.variation_name == Symbol
         
         if validated:
-            query = sql.and_(query,var_feature_table.c.validation_status!=None)
+            # in Release 65, the default validated status is now ''
+            # why?? thanks Ensembl!
+            null = None
+            if int(self.Release) >= 65:
+                null = ''
+                
+            query = sql.and_(query,var_feature_table.c.validation_status!=null)
         
         query = sql.select([var_feature_table],
                     query).order_by(var_feature_table.c.seq_region_start)
@@ -498,7 +540,11 @@ class Genome(object):
         Arguments:
             - property_type: valid values are biotype, status, effect"""
         property_type = property_type.lower()
-        db = [self.CoreDb, self.VarDb][property_type == 'effect']
+        if property_type == 'effect':
+            db = self.VarDb
+        else:
+            db = self.CoreDb
+        
         property_map = {'effect': ('variation_feature', 'consequence_type'),
                         'biotype': ('gene', 'biotype'),
                         'status': ('gene', 'status')}
