@@ -12,10 +12,10 @@ from cogent.db.ensembl.assembly import Coordinate, CoordSystem, \
 from cogent.db.ensembl.sequence import get_sequence
 
 __author__ = "Gavin Huttley"
-__copyright__ = "Copyright 2007-2011, The Cogent Project"
+__copyright__ = "Copyright 2007-2012, The Cogent Project"
 __credits__ = ["Gavin Huttley"]
 __license__ = "GPL"
-__version__ = "1.5.1"
+__version__ = "1.5.3"
 __maintainer__ = "Gavin Huttley"
 __email__ = "Gavin.Huttley@anu.edu.au"
 __status__ = "alpha"
@@ -29,8 +29,11 @@ _limit_words = lambda x : DisplayString(x, with_quotes=True, num_words=3)
 class _Region(LazyRecord):
     """a simple genomic region object"""
     Type = None
-    _attr_ensembl_table_map = None
-    _location_column_prefix = 'seq_region_'
+    
+    def __init__(self):
+        super(_Region, self).__init__()
+        self._attr_ensembl_table_map = None
+        self._location_column_prefix = 'seq_region_'
     
     def __len__(self):
         return len(self.Location)
@@ -224,7 +227,12 @@ class _StableRegion(GenericRegion):
         # subclasses need to provide a function for loading the correct
         # record for obtaining a stable_id
         table_name = self._attr_ensembl_table_map['StableId']
-        func_name = '_get_%s_record' % table_name
+        
+        if self.genome.GeneralRelease >= 65:
+            func_name = '_get_%s_record' % (table_name + '_stable_id')
+        else:
+            func_name = '_get_%s_record' % table_name
+        
         func = getattr(self, func_name)
         func()
         attr_column_map = [('StableId', 'stable_id', _quoted)]
@@ -263,19 +271,24 @@ class Gene(_StableRegion):
     """a gene region"""
     Type = 'gene'
     _member_types = ['Transcripts']
-    _attr_ensembl_table_map = dict(StableId='gene_stable_id', Symbol='xref',
-        Description='gene', BioType='gene', Location='gene',
-        CanonicalTranscript='gene',
-        Transcripts='transcript',
-        Exons='transcript')
     
     def __init__(self, genome, db, StableId=None, Symbol=None, Location=None, data=None):
         """constructed by a genome instance"""
         super(Gene, self).__init__(genome, db, Location=Location)
+        
+        self._attr_ensembl_table_map = dict(StableId=['gene_stable_id',
+                                                      'gene'][genome.GeneralRelease >= 65],
+                                            Symbol='xref',
+                                            Description='gene', BioType='gene', Location='gene',
+                                            CanonicalTranscript='gene',
+                                            Transcripts='transcript',
+                                            Exons='transcript')
+        
         if data is None:
             args = [dict(StableId=StableId), dict(Symbol=Symbol)][StableId is None]
             assert args
             data = asserted_one(list(self.genome._get_gene_query(db, **args).execute()))
+        
         for name, func in \
                     [('StableId',self._get_gene_stable_id_record),
                      ('BioType', self._get_gene_record),
@@ -315,7 +328,8 @@ class Gene(_StableRegion):
     def _get_gene_stable_id_record(self):
         """adds the gene_stable_id data to self._table_rows"""
         attr_column_map = [('StableId', 'stable_id', _quoted)]
-        self._populate_cache_from_record(attr_column_map,'gene_stable_id')
+        self._populate_cache_from_record(attr_column_map,
+                                        self._attr_ensembl_table_map['StableId'])
         return
     
     def _get_xref_record(self):
@@ -427,14 +441,17 @@ class Gene(_StableRegion):
 class Transcript(_StableRegion):
     Type = 'transcript'
     _member_types = ['Exons', 'TranslatedExons']
-    _attr_ensembl_table_map = dict(StableId='transcript_stable_id',
-                                   Location='transcript',
-                                   Status = 'transcript',
-                                   TranslatedExons='translation')
     
     def __init__(self, genome, db, transcript_id, data, Location=None):
         """created by Gene"""
         super(Transcript, self).__init__(genome, db, Location=Location)
+
+        self._attr_ensembl_table_map = dict(StableId=['transcript_stable_id',
+                                                'transcript'][genome.GeneralRelease >= 65],
+                                           Location='transcript',
+                                           Status = 'transcript',
+                                           TranslatedExons='translation')
+
         self._am_prot_coding = None
         self.transcript_id = transcript_id
         self._table_rows['transcript'] = data
@@ -531,6 +548,7 @@ class Transcript(_StableRegion):
         except NoItemError:
             self._set_null_values(['TranslatedExons'], 'translation')
             return
+        
         self._table_rows['translation'] = record
     
     def _get_transcript(self):
@@ -674,6 +692,16 @@ class Transcript(_StableRegion):
                 full_seq = exon.Seq
                 continue
             full_seq += exon.Seq
+        
+        # check first exon PhaseStart is 0 and last exon PhaseEnd
+        if exons[0].PhaseStart > 0:
+            fill = DNA.makeSequence('N' * exons[0].PhaseStart, Name=full_seq.Name)
+            full_seq = fill + full_seq
+        
+        if exons[-1].PhaseEnd > 0:
+            fill = DNA.makeSequence('N' * exons[-1].PhaseEnd, Name=full_seq.Name)
+            full_seq += fill
+        
         self._cached['Cds'] = full_seq
     
     def _get_cds(self):
@@ -695,8 +723,12 @@ class Transcript(_StableRegion):
             return
         
         DEBUG = False
+        # enforce multiple of 3
+        cds = self.Cds
+        length = len(cds)
+        cds = cds[: length - (length % 3)]
         try:
-            cds = self.Cds.withoutTerminalStopCodon()
+            cds = cds.withoutTerminalStopCodon()
         except AssertionError:
             if not DEBUG:
                 raise
@@ -795,11 +827,15 @@ class Transcript(_StableRegion):
 
 class Exon(_StableRegion):
     Type = 'exon'
-    _attr_ensembl_table_map = dict(StableId='exon_stable_id',
-                                    Location='exon')
+    
     def __init__(self, genome, db, exon_id, Rank, Location=None):
         """created by a Gene"""
         _StableRegion.__init__(self, genome, db, Location=Location)
+
+        self._attr_ensembl_table_map = dict(StableId=['exon_stable_id',
+                                                      'exon'][genome.GeneralRelease >= 65],
+                                            Location='exon')
+
         self.exon_id = exon_id
         self.Rank = Rank
     
@@ -814,12 +850,18 @@ class Exon(_StableRegion):
         return cmp(self.Rank, other.Rank)
     
     def _get_exon_stable_id_record(self):
-        exon_stable_id_table = self.db.getTable('exon_stable_id')
+        if self.genome.GeneralRelease >= 65:
+            # release >= 65, data is just in the exon table
+            self._get_exon_record()
+            return
+        
+        table_name = self._attr_ensembl_table_map['StableId']
+        exon_stable_id_table = self.db.getTable(table_name)
         query = sql.select([exon_stable_id_table.c.stable_id],
                            exon_stable_id_table.c.exon_id == self.exon_id)
         records = query.execute()
         record = asserted_one(records.fetchall())
-        self._table_rows['exon_stable_id'] = record
+        self._table_rows[table_name] = record
     
     def _get_exon_record(self):
         # this will be called by _Region parent class to make the location
@@ -837,6 +879,26 @@ class Exon(_StableRegion):
     
     Symbol = property(_get_symbol)
     
+    def _make_phase(self):
+        """creates the exon phase attributes"""
+        if 'exon' not in self._table_rows:
+            self._get_exon_record()
+        
+        exon = self._table_rows['exon']
+        self._cached['PhaseStart'] = exon['phase']
+        self._cached['PhaseEnd'] = exon['end_phase']
+    
+    @property
+    def PhaseStart(self):
+        """reading frame start for this exon"""
+        return self._get_cached_value('PhaseStart', self._make_phase)
+    
+    @property
+    def PhaseEnd(self):
+        """reading frame end for this exon"""
+        return self._get_cached_value('PhaseEnd', self._make_phase)
+    
+    
 
 
 class Intron(GenericRegion):
@@ -879,15 +941,6 @@ def _set_to_string(val):
 class Variation(_Region):
     """genomic variation"""
     Type = 'variation'
-    _attr_ensembl_table_map = dict(Effect='variation_feature',
-                                Symbol='variation_feature',
-                                Validation='variation_feature',
-                                MapWeight='variation_feature',
-                                FlankingSeq='flanking_sequence',
-                                PeptideAlleles='transcript_variation',
-                                TranslationLocation='transcript_variation',
-                                Location='variation_feature',
-                                AlleleFreqs='allele')
     
     def __init__(self, genome, db=None, Effect=None, Symbol=None, data=None):
         self.genome = genome
@@ -897,9 +950,23 @@ class Variation(_Region):
         self.transcript_variation_table = get_table('transcript_variation')
         self.flanking_sequence_table = get_table('flanking_sequence')
         self.allele_table = get_table('allele')
+        try:
+            self.allele_code_table = get_table('allele_code')
+        except sql.exceptions.ProgrammingError:
+            self.allele_code_table = None
         
         super(Variation, self).__init__()
         
+        self._attr_ensembl_table_map = dict(Effect='variation_feature',
+                                            Symbol='variation_feature',
+                                            Validation='variation_feature',
+                                            MapWeight='variation_feature',
+                                            FlankingSeq='flanking_sequence',
+                                            PeptideAlleles='transcript_variation',
+                                            TranslationLocation='transcript_variation',
+                                            Location='variation_feature',
+                                            AlleleFreqs='allele')
+        
         assert data is not None, 'Variation record created in an unusual way'
         for name, value, func in \
                 [('Effect',Effect,self._get_variation_table_record),
@@ -922,7 +989,11 @@ class Variation(_Region):
             (my_type, self.Symbol, self.Effect, self.Alleles)
     
     def _get_variation_table_record(self):
-        attr_name_map = [('Effect', 'consequence_type', _set_to_string),
+        consequence_type = 'consequence_type'
+        if self.genome.GeneralRelease > 67:
+            consequence_type += 's' # change to plural column name
+        
+        attr_name_map = [('Effect', consequence_type, _set_to_string),
                          ('Alleles', 'allele_string', _quoted),
                          ('Symbol', 'variation_name', _quoted),
                          ('Validation', 'validation_status', _set_to_string),
@@ -998,9 +1069,25 @@ class Variation(_Region):
             self._cached[('AlleleFreqs')] = self.NULL_VALUE
             return
         
+        # property change from >= 65, allele ids need to be looked up in
+        # the allele_code table
+        allele_code = self.allele_code_table
+        
         self._table_rows['allele_table'] = records
-        data = [(rec['allele'], rec['frequency'], rec['sample_id'])
-                                    for rec in records if rec['sample_id']]
+        data = []
+        for rec in records:
+            if not rec['sample_id']:
+                continue
+            
+            if allele_code is None:
+                allele = rec['allele']
+            else:
+                allele_query = sql.select([allele_code.c.allele],
+                                allele_code.c.allele_code_id == rec['allele_code_id'])
+                allele = list(allele_query.execute())[0][0]
+            
+            data.append((allele, rec['frequency'], rec['sample_id']))
+        
         if not data:
             self._cached[('AlleleFreqs')] = self.NULL_VALUE
             return
@@ -1152,13 +1239,14 @@ class CpGisland(GenericRegion):
 
 class Repeat(GenericRegion):
     Type = 'repeat'
-    _attr_ensembl_table_map = dict(Symbol='repeat_consensus',
-                                    RepeatType='repeat_consensus',
-                                    RepeatClass='repeat_consensus',
-                                    Consensus='repeat_consensus')
     
     def __init__(self, genome, db, Location, Score, data):
         super(Repeat, self).__init__(genome=genome, db=db, Location=Location)
+        self._attr_ensembl_table_map = dict(Symbol='repeat_consensus',
+                                            RepeatType='repeat_consensus',
+                                            RepeatClass='repeat_consensus',
+                                            Consensus='repeat_consensus')
+
         self.Score = Score
         # assume always created from repeat_feature table
         self._table_rows['repeat_feature']= data
