@@ -10,10 +10,10 @@ from cogent.db.ensembl.assembly import location_query
 from cogent.db.ensembl.species import Species
 
 __author__ = "Gavin Huttley"
-__copyright__ = "Copyright 2007-2011, The Cogent Project"
+__copyright__ = "Copyright 2007-2012, The Cogent Project"
 __credits__ = ["Gavin Huttley"]
 __license__ = "GPL"
-__version__ = "1.5.1"
+__version__ = "1.5.3"
 __maintainer__ = "Gavin Huttley"
 __email__ = "Gavin.Huttley@anu.edu.au"
 __status__ = "alpha"
@@ -331,7 +331,7 @@ class SyntenicRegions(_RelatedRegions):
                 name = str(loc)
             
             annotations[name] = seq.data.annotations
-            seq.Name = name
+            seq.Name = seq.data.Name = name
             seqs += [(name, seq)]
         
         if seqs is None:
@@ -345,11 +345,6 @@ class SyntenicRegions(_RelatedRegions):
         if omit_redundant:
             aln = aln.filtered(lambda x: set(x) != set('-'))
         
-        # the following is necessary because of a bug in the filtered code,
-        # which doesn't handle annotations correctly
-        if feature_types:
-            for name in annotations.keys():
-                aln.getSeq(name).annotations = annotations[name]
         return aln
     
 
