@@ -21,11 +21,11 @@ from cogent.align.pairwise import AlignableSeq
 from cogent.util.warning import discontinued, deprecated
 
 __author__ = "Peter Maxwell"
-__copyright__ = "Copyright 2007-2011, The Cogent Project"
+__copyright__ = "Copyright 2007-2012, The Cogent Project"
 __credits__ = ["Andrew Butterfield", "Peter Maxwell", "Gavin Huttley",
                     "Helen Lindsay"]
 __license__ = "GPL"
-__version__ = "1.5.1"
+__version__ = "1.5.3"
 __maintainer__ = "Gavin Huttley"
 __email__ = "gavin.huttley@anu.ed.au"
 __status__ = "Production"
@@ -138,7 +138,7 @@ class _LikelihoodParameterController(_LF):
         counts += pseudocount
         mprobs = counts/(1.0*sum(counts))
         self.setMotifProbs(mprobs, locus=locus, is_constant=is_constant, 
-                is_independent=is_independent, auto=auto)
+                is_independent=is_independent, auto=auto, **kwargs)
     
     def setMotifProbs(self, motif_probs, locus=None, bin=None, is_constant=None, 
                 is_independent=None, auto=False, **kwargs):
@@ -151,7 +151,7 @@ class _LikelihoodParameterController(_LF):
             is_constant = not self.optimise_motif_probs
         self.model.setParamControllerMotifProbs(self, motif_probs, 
             is_constant=is_constant, bin=bin, locus=locus, 
-            is_independent=is_independent)
+            is_independent=is_independent, **kwargs)
         if not auto:
             self.mprobs_from_alignment = False  # should be done per-locus
     
