@@ -1,11 +1,13 @@
 #!/usr/bin/env python
 import os
+import warnings
+from cogent.parse.record import FileFormatError
 
 __author__ = "Peter Maxwell and Gavin Huttley"
-__copyright__ = "Copyright 2007-2011, The Cogent Project"
+__copyright__ = "Copyright 2007-2012, The Cogent Project"
 __credits__ = ["Peter Maxwell", "Gavin Huttley"]
 __license__ = "GPL"
-__version__ = "1.5.1"
+__version__ = "1.5.3"
 __maintainer__ = "Gavin Huttley"
 __email__ = "gavin.huttley@anu.edu.au"
 __status__ = "Production"
@@ -16,6 +18,9 @@ def save_to_filename(alignment, filename, format, **kw):
             - filename: name of the sequence alignment file
             - format: the multiple sequence file format
     """
+    if format is None:
+        raise FileFormatError("format not known")
+    
     f = open(filename, 'w')
     try:
         write_alignment_to_file(f, alignment, format, **kw)
