@@ -14,16 +14,16 @@ GlobalOptimiser = SimulatedAnnealing
 LocalOptimiser = Powell
 
 __author__ = "Peter Maxwell"
-__copyright__ = "Copyright 2007-2011, The Cogent Project"
+__copyright__ = "Copyright 2007-2012, The Cogent Project"
 __credits__ = ["Andrew Butterfield", "Peter Maxwell", "Gavin Huttley"]
 __license__ = "GPL"
-__version__ = "1.5.1"
+__version__ = "1.5.3"
 __maintainer__ = "Gavin Huttley"
 __email__ = "gavin.huttley@anu.edu.au"
 __status__ = "Production"
 
 def unsteadyProgressIndicator(display_progress, label='', start=0.0, end=1.0):
-    template = 'f = % #10.6g  ±  % 9.3e   evals = %6i '
+    template = u'f = % #10.6g  ±  % 9.3e   evals = %6i '
     label = label.rjust(5)
     goal = [1.0e-20]
     def _display_progress(remaining, *args):
