@@ -8,10 +8,10 @@ from cogent.maths.scipy_optimize import fmin_powell
 import cogent.maths.stats.rarefaction as rarefaction
 
 __author__ = "Rob Knight"
-__copyright__ = "Copyright 2007-2011, The Cogent Project"
+__copyright__ = "Copyright 2007-2012, The Cogent Project"
 __credits__ = ["Rob Knight", "Justin Kuczynski"]
 __license__ = "GPL"
-__version__ = "1.5.1"
+__version__ = "1.5.3"
 __maintainer__ = "Rob Knight"
 __email__ = "rob@spot.colorado.edu"
 __status__ = "Production"
@@ -78,6 +78,10 @@ def reciprocal_simpson(counts):
     """1/Simpson's index"""
     return 1.0/simpson(counts)
 
+def simpson_reciprocal(counts):
+    """1/D (1/dominance)"""
+    return 1.0/dominance(counts)
+
 def shannon(counts, base=2):
     """Returns Shannon entropy of counts, default in bits."""
     freqs = counts/float(counts.sum())
