@@ -7,7 +7,7 @@ import sys
 from time import clock
 from datetime import datetime
 from string import maketrans, strip
-from random import randrange, choice
+from random import randrange, choice, randint
 from sys import maxint
 from os import popen, remove, makedirs, getenv
 from os.path import join, abspath, exists, isdir
@@ -19,12 +19,12 @@ import hashlib
 from cogent.util.option_parsing import parse_command_line_parameters
 
 __author__ = "Rob Knight"
-__copyright__ = "Copyright 2007-2011, The Cogent Project"
+__copyright__ = "Copyright 2007-2012, The Cogent Project"
 __credits__ = ["Rob Knight", "Peter Maxwell", "Amanda Birmingham",
                     "Sandra Smit", "Zongzhi Liu", "Daniel McDonald",
                     "Kyle Bittinger", "Marcin Cieslik"]
 __license__ = "GPL"
-__version__ = "1.5.1"
+__version__ = "1.5.3"
 __maintainer__ = "Rob Knight"
 __email__ = "rob@spot.colorado.edu"
 __status__ = "Production"
@@ -1463,3 +1463,101 @@ def get_random_directory_name(suppress_mkdir=False,\
     if return_absolute_path:
         return abs_dirpath
     return dirpath
+
+def get_independent_coords(spans, random_tie_breaker=False):
+    """returns non-overlapping spans. spans must have structure
+        [(start, end, ..), (..)]. spans can be decorated with arbitrary data
+        after the end entry.
+    
+    Arguments:
+        - random_tie_breaker: break overlaps by randomly choosing the first
+          or second span. Defaults to the first span.
+    """
+    
+    if len(spans) <= 1:
+        return spans
+    
+    last = spans[0]
+    result = [last]
+    for i in range(1, len(spans)):
+        curr = spans[i]
+        if curr[0] < last[1]:
+            if random_tie_breaker:
+                result[-1] = [last, curr][randint(0,1)]
+            else:
+                result[-1] = last
+            continue
+        
+        result.append(curr)
+        last = curr
+    
+    return result
+
+def get_merged_overlapping_coords(start_end):
+    """merges overlapping spans, assumes sorted by start"""
+    result = [start_end[0]]
+    prev_end = result[0][-1]
+    for i in range(1, len(start_end)):
+        curr_start, curr_end = start_end[i]
+        # if we're beyond previous, add and continue
+        if curr_start > prev_end:
+            prev_end = curr_end
+            result.append([curr_start, curr_end])
+        elif curr_end > prev_end:
+            prev_end = curr_end
+            result[-1][-1] = prev_end
+        else:
+            pass # we lie completely within previous span
+    
+    return result
+
+def get_run_start_indices(values, digits=None, converter_func=None):
+    """returns starting index, value for all distinct values"""
+    assert not (digits and converter_func), \
+        'Cannot set both digits and converter_func'
+    
+    if digits is not None:
+        converter_func = lambda x: round(x, digits)
+    elif converter_func is None:
+        converter_func = lambda x: x
+    
+    last_val = None
+    for index, val in enumerate(values):
+        val = converter_func(val)
+        if val != last_val:
+            yield [index, val]
+        
+        last_val = val
+    
+    return
+
+def get_merged_by_value_coords(spans_value, digits=None):
+    """returns adjacent spans merged if they have the same value. Assumes
+    [(start, end, val), ..] structure and that spans_value is sorted in
+    ascending order.
+    
+    Arguments:
+        - digits: if None, any data can be handled and exact values are
+          compared. Otherwise values are rounded to that many digits.
+    """
+    assert len(spans_value[0]) == 3, 'spans_value must have 3 records per row'
+    
+    starts, ends, vals = zip(*spans_value)
+    indices_distinct_vals = get_run_start_indices(vals, digits=digits)
+    data = []
+    i = 0
+    for index, val in indices_distinct_vals:
+        start = starts[index]
+        end = ends[index]
+        prev_index = max(index-1, 0)
+        try:
+            data[-1][1] = ends[prev_index]
+        except IndexError:
+            pass
+        
+        data.append([start, end, val])
+    
+    if index < len(ends):
+        data[-1][1] = ends[-1]
+    
+    return data
