@@ -28,10 +28,10 @@ import itertools
 from cogent.util import parallel, terminal
 
 __author__ = "Peter Maxwell"
-__copyright__ = "Copyright 2007-2011, The Cogent Project"
+__copyright__ = "Copyright 2007-2012, The Cogent Project"
 __credits__ = ["Peter Maxwell"]
 __license__ = "GPL"
-__version__ = "1.5.1"
+__version__ = "1.5.3"
 
 try:
     curses_terminal = terminal.CursesOutput()
@@ -240,7 +240,8 @@ class LogFileOutput(object):
             delta = '+%s' % int(time.time() - self.t0)
             progress = int(100*progress+0.5)
             print >>self.output, "%s %5s %3i%% %s" % (
-                    self.lpad, delta, progress, message)
+                    self.lpad, delta, progress,
+                    str(message.encode('utf8')))
             
 
 class CursesTerminalProgressBar(object):
@@ -284,8 +285,9 @@ class CursesTerminalProgressBar(object):
         
         if progress is not None:
             self.stderr.writelines([bar, '\n'])
+        
         if message is not None:
-            self.stderr.writelines([message[:width], '\n'])
+            self.stderr.writelines([str(message[:width].encode('utf8')), u'\n'])
         self.line_count = (progress is not None) + (message is not None)
 
 
