@@ -12,15 +12,15 @@ import cPickle, csv
 from gzip import GzipFile
 
 import numpy
-from cogent.format import table as table_format
+from cogent.format import table as table_format, bedgraph
 
 from cogent.util.dict_array import DictArray
 
 __author__ = "Gavin Huttley"
-__copyright__ = "Copyright 2007-2011, The Cogent Project"
+__copyright__ = "Copyright 2007-2012, The Cogent Project"
 __credits__ = ["Gavin Huttley", "Felix Schill"]
 __license__ = "GPL"
-__version__ = "1.5.1"
+__version__ = "1.5.3"
 __maintainer__ = "Gavin Huttley"
 __email__ = "gavin.huttley@anu.edu.au"
 __status__ = "Production"
@@ -142,6 +142,41 @@ class Table(DictArray):
         # on args
         self._column_templates = column_templates or {}
     
+    def __repr__(self):
+        row = []
+        try:
+            for val in self.array[0][:3]:
+                if isinstance(val, float):
+                    row.append('%.4f' % val)
+                elif isinstance(val, int):
+                    row.append('%d' % val)
+                else:
+                    row.append('%r' % val)
+        except IndexError:
+            # an empty table
+            pass
+        
+        row_trunc = ', '.join(row)
+        header_trunc = ', '.join(map(repr, self.Header[:3]))
+        if self.Shape[1] > 3:
+            header_trunc = '[%s,..]' % header_trunc
+            row_trunc = '[%s,..]' % row_trunc
+        else:
+            header_trunc = '[%s]' % header_trunc
+            row_trunc = '[%s]' % row_trunc
+        
+        if self.Shape[0] > 1:
+            row_trunc = '[%s,..]' % row_trunc
+        else:
+            row_trunc = '[[%s]]' % row_trunc
+        
+        if self.Shape[0] == 0:
+            row_trunc = str([])
+        
+        result = 'Table(numrows=%s, numcols=%s, header=%s, rows=%s)' % \
+            (self.Shape[0], self.Shape[1], header_trunc, row_trunc)
+        return result
+    
     def __str__(self):
         return self.tostring()
     
@@ -236,14 +271,17 @@ class Table(DictArray):
         
         self._column_templates[column_head] = format_template
     
-    def tostring(self, borders = True, sep = None, format = '', **kwargs):
+    def tostring(self, borders=True, sep=None, format='', **kwargs):
         """Return the table as a formatted string.
         
         Arguments:
-            - format: possible formats are 'rest', 'latex', 'html', 'phylip', or
-              simple text (default)
+            - format: possible formats are 'rest', 'latex', 'html', 'phylip',
+              'bedgraph', or simple text (default).
             - sep: A string separator for delineating columns, e.g. ',' or '\t'.
               Overrides format.
+        
+        NOTE: If format is bedgraph, assumes that column headers are chrom,
+        start, end, value. In that order!
         """
         if format.lower() == 'phylip':
             missing_data = "%.4f" % 0.0
@@ -252,13 +290,14 @@ class Table(DictArray):
         
         # convert self to a 2D list
         formatted_table = self.array.tolist()
-        header, formatted_table = table_format.formattedCells(formatted_table,
+        if format != 'bedgraph':
+            header, formatted_table = table_format.formattedCells(formatted_table,
                                     self.Header,
                                     digits = self._digits,
                                     column_templates = self._column_templates,
                                     missing_data = missing_data)
-        args = (header, formatted_table, self.Title, self.Legend)
-        if sep:
+            args = (header, formatted_table, self.Title, self.Legend)
+        if sep and format != 'bedgraph':
             return table_format.separatorFormat(*args + (sep,))
         elif format == 'rest':
             return table_format.gridTableFormat(*args)
@@ -276,6 +315,12 @@ class Table(DictArray):
             formatted_table = [row[self._row_ids:] for row in formatted_table]
             header = header[self._row_ids:]
             return table_format.phylipMatrix(formatted_table, header)
+        elif format == 'bedgraph':
+            assert self.Shape[1] == 4, 'bedgraph format is for 4 column tables'
+            # assuming that header order is chrom, start, end, val
+            formatted_table = bedgraph.bedgraph(self.sorted().array.tolist(),
+                **kwargs)
+            return formatted_table
         else:
             return table_format.simpleFormat(*args + (self._max_width,
                                 self._row_ids, borders, self.Space))
@@ -352,7 +397,7 @@ class Table(DictArray):
         elif format == 'pickle':
             data = self.__getstate__()
             cPickle.dump(data, outfile)
-        elif sep is not None:
+        elif sep is not None and format != 'bedgraph':
             writer = csv.writer(outfile, delimiter = sep)
             if self.Title:
                 writer.writerow([self.Title])
@@ -365,6 +410,10 @@ class Table(DictArray):
             outfile.writelines(table + '\n')
         outfile.close()
     
+    def toBedgraph(self, chrom_col, start_col, end_col):
+        """docstring for toBedgraph"""
+        pass
+    
     def appended(self, new_column, *tables, **kwargs):
         """Append an arbitrary number of tables to the end of this one.
         Returns a new table object. Optional keyword arguments to the new
