@@ -7,24 +7,20 @@ from cogent.core.annotation import Feature, Variable
 from cogent.core.location import Map, Span
 
 __author__ = "Gavin Huttley"
-__copyright__ = "Copyright 2007-2011, The Cogent Project"
+__copyright__ = "Copyright 2007-2012, The Cogent Project"
 __credits__ = ["Gavin Huttley"]
 __license__ = "GPL"
-__version__ = "1.5.1"
+__version__ = "1.5.3"
 __maintainer__ = "Gavin Huttley"
 __email__ = "gavin.huttley@anu.edu.au"
 __status__ = "Production"
 
-def makeSampleSequence(mid_gaps=False):
+def makeSampleSequence(with_gaps=False):
     raw_seq = 'AACCCAAAATTTTTTGGGGGGGGGGCCCC'
     cds = (15, 25)
     utr = (12, 15)
-    if mid_gaps:
-        rev_seq = raw_seq[:5] + '-----' +raw_seq[10:]
-        raw_seq = rev_seq
-        # annotations only make sense when they're on the raw sequence
-        cds = (10, 20)
-        utr = (5, 8)
+    if with_gaps:
+        raw_seq = raw_seq[:5] + '-----' +raw_seq[10:-2] + '--'
     seq = DNA.makeSequence(raw_seq)
     seq.addAnnotation(Feature, 'CDS', 'CDS', [cds])
     seq.addAnnotation(Feature, "5'UTR", "5' UTR", [utr])
@@ -32,7 +28,7 @@ def makeSampleSequence(mid_gaps=False):
 
 def makeSampleAlignment():
     seq1 = makeSampleSequence()
-    seq2 = makeSampleSequence(mid_gaps=True)
+    seq2 = makeSampleSequence(with_gaps=True)
     seqs = {'FAKE01': seq1, 'FAKE02': seq2}
     aln = LoadSeqs(data = seqs)
     aln.addAnnotation(Feature, 'misc_feature', 'misc', [(12,25)])
@@ -104,7 +100,19 @@ class TestAnnotations(unittest.TestCase):
                 new = str(list(newaln.getAnnotationsFromSequence(name,
                                                     annot_type))[0].getSlice())
                 assert orig == new, (name, annot_type, orig, new)
-    
+
+    def test_feature_projection(self):
+        expecteds = {"FAKE01": "CCCAAAATTTTTT", "FAKE02": "CCC-----TTTTT"}
+        aln_ltr = self.aln.getAnnotationsMatching('LTR')[0]
+        for seq_name in ['FAKE01', 'FAKE02']:
+            expected = expecteds[seq_name]
+            seq_ltr = self.aln.projectAnnotation(seq_name, aln_ltr)
+            if '-' in expected:
+                self.assertRaises(ValueError, seq_ltr.getSlice)
+                seq_ltr = seq_ltr.withoutLostSpans()
+                expected = expected.replace('-', '')
+            self.assertEqual(seq_ltr.getSlice(), expected)
+        
     def test_reversecomplement(self):
         """test correct translation of annotations on reverse complement."""
         aln_expecteds = {"misc_feature":{'FAKE01': 'TTTGGGGGGGGGG',
