@@ -5,12 +5,13 @@ import unittest, os, tempfile
 
 from cogent import DNA, RNA, STANDARD_CODON as CODON, PROTEIN, Sequence, \
                 LoadSeqs
+from cogent.parse.record import FileFormatError
 
 __author__ = "Peter Maxwell, Gavin Huttley and Rob Knight"
-__copyright__ = "Copyright 2007-2011, The Cogent Project"
+__copyright__ = "Copyright 2007-2012, The Cogent Project"
 __credits__ = ["Peter Maxwell", "Gavin Huttley", "Rob Knight"]
 __license__ = "GPL"
-__version__ = "1.5.1"
+__version__ = "1.5.3"
 __maintainer__ = "Gavin Huttley"
 __email__ = "gavin.huttley@anu.edu.au"
 __status__ = "Production"
@@ -32,7 +33,17 @@ class ReadingWritingFileFormats(unittest.TestCase):
             fn = tempfile.mktemp(suffix='.'+suffix)
             aln.writeToFile(filename=fn)
             os.remove(fn)
-        
+    
+    def test_write_unknown_raises(self):
+        """writing unknown format raises FileFormatError"""
+        filename = os.path.join(data_path, "primates_brca1.fasta")
+        aln = LoadSeqs(filename)
+        self.assertRaises(FileFormatError, aln.writeToFile, filename='blah')
+        self.assertRaises(FileFormatError, aln.writeToFile,
+                filename='blah.txt')
+        self.assertRaises(FileFormatError, aln.writeToFile,
+                filename='blah.fasta', format='noway')
+    
     def test_fasta(self):
         self._loadfromfile("formattest.fasta")
         
@@ -260,6 +271,12 @@ class AlignmentTestMethods(unittest.TestCase):
         coll_rc = coll.rc()
         self.assertEqual(coll_rc.todict(), dna_rc)
     
+    def test_reversecomplement_with_ambig(self):
+        """correctly reverse complement with ambiguous bases"""
+        n = LoadSeqs(data=[['x', '?-???AA'], ['y', '-T----T']], moltype=DNA)
+        rc = n.rc()
+        self.assertEqual(rc.todict(), {'x': 'TT???-?', 'y': 'A----A-'})
+    
     def test_getasdict(self):
         """getting the alignment as a dictionary"""
         seqs={'seq1': 'ACGT--GT', 'seq2': 'ACGTACGT', 'seq3': 'ACGTACGT'}
