@@ -11,12 +11,13 @@ from cogent.util.unit_test import TestCase, main
 from numpy import array, arange
 
 __author__ = "Rob Knight"
-__copyright__ = "Copyright 2007-2011, The Cogent Project"
+__copyright__ = "Copyright 2007-2012, The Cogent Project"
 __credits__ = ["Rob Knight", "Catherine Lozupone", "Daniel McDonald",
                "Peter Maxwell", "Gavin Huttley", "Andrew Butterfield",
-               "Matthew Wakefield", "Justin Kuczynski","Jens Reeder"]
+               "Matthew Wakefield", "Justin Kuczynski","Jens Reeder",
+               "Jose Carlos Clemente Litran"]
 __license__ = "GPL"
-__version__ = "1.5.1"
+__version__ = "1.5.3"
 __maintainer__ = "Rob Knight"
 __email__ = "rob@spot.colorado.edu"
 __status__ = "Production"
@@ -154,6 +155,61 @@ class TreeNodeTests(TestCase):
         self.assertEqual(self.BigParent.getNewick(), \
                 '(0,1,2,3,4,5,6,7,8,(a,b,c)9)x;')
 
+    def test_multifurcating(self):
+        """Coerces nodes to have <= n children"""
+        t_str = "((a:1,b:2,c:3)d:4,(e:5,f:6,g:7)h:8,(i:9,j:10,k:11)l:12)m:14;"
+        t = DndParser(t_str)
+        
+        # can't break up easily... sorry 80char
+        exp_str = "((a:1.0,(b:2.0,c:3.0):0.0)d:4.0,((e:5.0,(f:6.0,g:7.0):0.0)h:8.0,(i:9.0,(j:10.0,k:11.0):0.0)l:12.0):0.0)m:14.0;"
+        obs = t.multifurcating(2)
+        self.assertEqual(obs.getNewick(with_distances=True), exp_str)
+        self.assertNotEqual(t.getNewick(with_distances=True),
+                            obs.getNewick(with_distances=True))
+
+        obs = t.multifurcating(2, 0.5)
+        exp_str = "((a:1.0,(b:2.0,c:3.0):0.5)d:4.0,((e:5.0,(f:6.0,g:7.0):0.5)h:8.0,(i:9.0,(j:10.0,k:11.0):0.5)l:12.0):0.5)m:14.0;"
+        self.assertEqual(obs.getNewick(with_distances=True), exp_str)
+
+        t_str = "((a,b,c)d,(e,f,g)h,(i,j,k)l)m;"
+        exp_str = "((a,(b,c))d,((e,(f,g))h,(i,(j,k))l))m;"
+        t = DndParser(t_str, constructor=TreeNode)
+        obs = t.multifurcating(2)
+        self.assertEqual(obs.getNewick(with_distances=True), exp_str)
+        obs = t.multifurcating(2, eps=10) # no effect on TreeNode type
+        self.assertEqual(obs.getNewick(with_distances=True), exp_str)
+
+        self.assertRaises(TreeError, t.multifurcating, 1)
+
+    def test_multifurcating_nameunnamed(self):
+        """Coerces nodes to have <= n children"""
+        t_str = "((a:1,b:2,c:3)d:4,(e:5,f:6,g:7)h:8,(i:9,j:10,k:11)l:12)m:14;"
+        t = DndParser(t_str)
+        
+        exp_str = "((a:1.0,(b:2.0,c:3.0):0.0)d:4.0,((e:5.0,(f:6.0,g:7.0):0.0)h:8.0,(i:9.0,(j:10.0,k:11.0):0.0)l:12.0):0.0)m:14.0;"
+        obs = t.multifurcating(2, name_unnamed=True)
+
+        c0,c1 = obs.Children
+        self.assertTrue(c0.Children[1].Name.startswith('AUTO'))
+        self.assertTrue(c1.Name.startswith('AUTO'))
+        self.assertTrue(c1.Children[0].Children[1].Name.startswith('AUTO'))
+        self.assertTrue(c1.Children[1].Children[1].Name.startswith('AUTO'))
+        self.assertEqual(len(c0.Children[1].Name), 22)
+        self.assertEqual(len(c1.Name), 22)
+        self.assertEqual(len(c1.Children[0].Children[1].Name), 22)
+        self.assertEqual(len(c1.Children[1].Children[1].Name), 22)
+        names = [n.Name for n in t.nontips()]
+        self.assertEqual(len(names), len(set(names)))
+
+    def test_bifurcating(self):
+        """Coerces nodes to have <= 2 children"""
+        t_str = "((a:1,b:2,c:3)d:4,(e:5,f:6,g:7)h:8,(i:9,j:10,k:11)l:12)m:14;"
+        t = DndParser(t_str)
+        
+        # can't break up easily... sorry 80char
+        exp_str = "((a:1.0,(b:2.0,c:3.0):0.0)d:4.0,((e:5.0,(f:6.0,g:7.0):0.0)h:8.0,(i:9.0,(j:10.0,k:11.0):0.0)l:12.0):0.0)m:14.0;"
+        obs = t.bifurcating()
+
     def test_cmp(self):
         """TreeNode cmp should compare using id"""
         nodes = self.TreeNode
@@ -856,6 +912,14 @@ class TreeNodeTests(TestCase):
         self.assertEqual(obs3, exp3)
         self.assertEqual(obs4, exp4)
 
+        # verify multiple calls work
+        t_mul = t1.copy()
+        exp_1 = t_mul.getNodeMatchingName('d')
+        exp_2 = t_mul.getNodeMatchingName('i')
+        obs_1 = t_mul.lowestCommonAncestor(['b','c'])
+        obs_2 = t_mul.lowestCommonAncestor(['g','h'])
+        self.assertEqual(obs_1, exp_1)
+        self.assertEqual(obs_2, exp_2)
 
     def test_lastCommonAncestor(self):
         """TreeNode LastCommonAncestor should provide last common ancestor"""
@@ -1869,9 +1933,15 @@ class TestTree(TestCase):
         self.tree.asciiArt()
         # unlabeled internal node
         tr = DndParser("(B:0.2,(C:0.3,D:0.4):0.6)F;")
-        tr.asciiArt(show_internal=True, compact=False)
-        tr.asciiArt(show_internal=True, compact=True)
-        tr.asciiArt(show_internal=False, compact=False)
+        obs = tr.asciiArt(show_internal=True, compact=False)
+        exp = """          /-B\n-F-------|\n         |          /-C\n          \\--------|\n                    \\-D"""
+        self.assertEqual(obs, exp)
+        obs = tr.asciiArt(show_internal=True, compact=True)
+        exp = """-F------- /-B\n          \-------- /-C\n                    \-D"""
+        self.assertEqual(obs, exp)
+        obs = tr.asciiArt(show_internal=False, compact=False)
+        exp = """          /-B\n---------|\n         |          /-C\n          \\--------|\n                    \\-D"""
+        self.assertEqual(obs, exp)
 
 # the following class repeats the above tests but using a big tree and big data-set
 class BigTreeSingleTests(TestTree):
