@@ -9,19 +9,23 @@ from cogent.db.ensembl.sequence import _assemble_seq
 from cogent.db.ensembl.util import asserted_one
 
 __author__ = "Gavin Huttley, Hua Ying"
-__copyright__ = "Copyright 2007-2011, The Cogent Project"
+__copyright__ = "Copyright 2007-2012, The Cogent Project"
 __credits__ = ["Gavin Huttley", "hua Ying"]
 __license__ = "GPL"
-__version__ = "1.5.1"
+__version__ = "1.5.3"
 __maintainer__ = "Gavin Huttley"
 __email__ = "Gavin.Huttley@anu.edu.au"
 __status__ = "alpha"
 
-Release = 62
+Release = 67
 
 if 'ENSEMBL_ACCOUNT' in os.environ:
-    host, username, password = os.environ['ENSEMBL_ACCOUNT'].split()
-    account = HostAccount(host, username, password)
+    args = os.environ['ENSEMBL_ACCOUNT'].split()
+    host, username, password = args[0:3]
+    kwargs = {}
+    if len(args) > 3:
+        kwargs['port'] = int(args[3])
+    account = HostAccount(host, username, password, **kwargs)
 else:
     account = get_ensembl_account(release=Release)
 
@@ -126,6 +130,12 @@ class TestGenome(GenomeTestBase):
         'GAGGTGGAGCAGCTGAAGAAAGAAGTGAAAAACACAAGAATTCCGATTTCCAAAGCGGGAAAGGAAAT'
         'CAAAGAGTACGTGGAGGCCCAAGCAGGAAACGATCCTTTTCTCAAAGGCATCCCTGAGGACAAGAATC'
         'CCTTCAAGGAGAAAGGTGGCTGTCTGATAAGCTGA')
+    
+    def test_get_distinct_biotype(self):
+        """Genome instance getDistinct should work on all genomes"""
+        for genome in self.gorilla, self.human, self.mouse, self.rat, self.macaq:
+            biotypes = genome.getDistinct('biotype')
+        
 
 
 class TestGene(GenomeTestBase):
@@ -168,26 +178,52 @@ class TestGene(GenomeTestBase):
 
     def test_failed_ensembl_annotation(self):
         """we demonstrate a failed annotation by ensembl"""
-        # I'm including this to demonstrate that Ensembl coords aren't
-        # always right. This case has a macaque gene which we correctly
+        # I'm including this to demonstrate that Ensembl coords are
+        # complex. This case has a macaque gene which we correctly
         # infer the CDS boundaries for according to Ensembl, but the CDS
-        # length is not divisible by 3, hence our code fails.
+        # length is not divisible by 3.
         gene = self.macaq.getGeneByStableId(StableId='ENSMMUG00000001551')
         transcript = gene.getMember('ENSMMUT00000002194')
-        try:
-            # this fails because of the above length issue
-            prot_seq = transcript.ProteinSeq
-        except AssertionError:
-            pass
-        # now if you slice the CDS to be divisible by 3, you can get the same
-        # protein sequence as that at Ensembl
-        l = 3 * (transcript.getCdsLength() / 3)
-        trunc_cds = transcript.Cds[:l]
+        # the following works because we enforce the length being divisble by 3
+        # in producing ProteinSeq
+        prot_seq = transcript.ProteinSeq
+        # BUT if you work off the Cds you will need to slice the CDS to be
+        # divisible by 3 to get the same protein sequence
+        l = transcript.getCdsLength()
+        trunc_cds = transcript.Cds[: l - (l % 3)]
         prot_seq = trunc_cds.getTranslation()
         self.assertEquals(str(prot_seq),
             'MPSSPLRVAVVCSSNQNRSMEAHNILSKRGFSVRSFGTGTHVKLPGPAPDKPNVYDFKTT'\
                'YDQMYNDLLRKDKELYTQNGILHMLDRNKRIKPRPERFQNCKDLFDLILTCEERVY')
+    
+    def test_exon_phases(self):
+        """correctly identify phase for an exon"""
+        stable_id = 'ENSG00000171408'
+        gene = self.human.getGeneByStableId(StableId=stable_id)
+        exon1 = gene.Transcripts[1].Exons[0]
+        # first two bases of codon missing
+        self.assertEquals(exon1.PhaseStart, 2)
+        # last two bases of codon missing
+        self.assertEquals(exon1.PhaseEnd, 1)
+        # can translate the sequence if we take those into account
+        seq = exon1.Seq[1:-1].getTranslation()
+        self.assertEquals(str(seq), 'HMLSKVGMWDFDIFLFDRLTN')
+    
+    def test_cds_from_outofphase(self):
+        """return a translatable Cds sequence from out-of-phase start"""
+        # canonical transcript phase end_phase
+        # ENSG00000111729 ENST00000229332 -1 -1
+        # ENSG00000177151 ENST00000317450 0 -1
+        # ENSG00000249624 ENST00000433395 1 -1
+        # ENSG00000237276 ENST00000442385 2 -1
+        # ENSG00000167744 ENST00000301411 -1 0
 
+        canon_ids = 'ENSG00000111729 ENSG00000177151 ENSG00000237276 ENSG00000167744 ENSG00000251184'.split()
+        for index, stable_id in enumerate(canon_ids):
+            gene = self.human.getGeneByStableId(StableId=stable_id)
+            transcript = gene.CanonicalTranscript
+            prot_seq = transcript.ProteinSeq
+    
     def test_gene_transcripts(self):
         """should return multiple transcripts"""
         stable_id = 'ENSG00000012048'
@@ -265,7 +301,7 @@ class TestGene(GenomeTestBase):
         results = list(self.human.getGenesMatching(BioType='Mt_tRNA', like=False))
         self.assertEquals(len(results), 22)
         results = list(self.human.getGenesMatching(BioType='Mt_tRNA', like=True))
-        self.assertEquals(len(results), 602)
+        self.assertEquals(len(results), 607)
 
     def test_get_by_decsr_biotype(self):
         """combining the description and biotype should return a result"""
@@ -436,7 +472,7 @@ class TestVariation(GenomeTestBase):
         snp_status = [('rs94', False), ('rs90', True)]
         for symbol, status in snp_status:
             snp = list(self.human.getVariation(Symbol=symbol, validated=True))
-            self.assertEquals(snp!=[], status)
+            self.assertEquals(snp != [], status)
 
     def test_allele_freqs(self):
         """exercising getting AlleleFreq data"""
@@ -484,7 +520,7 @@ class TestFeatures(GenomeTestBase):
         self.assertContains(symbols, self.igf2.Symbol.lower())
 
     def test_other_genes(self):
-        """docstring for test_other_genes"""
+        """docstring for est_other_genes"""
         mouse = self.mouse.getRegion(CoordName='5', Start=150791005,
                                         End=150838512, Strand='-')
         rat = self.rat.getRegion(CoordName='12', Start=4282534, End=4324019,
