@@ -30,11 +30,11 @@ Nucleotide = substitution_model.Nucleotide
 MotifChange = predicate.MotifChange
 
 __author__ = "Peter Maxwell and Gavin Huttley"
-__copyright__ = "Copyright 2007-2011, The Cogent Project"
+__copyright__ = "Copyright 2007-2012, The Cogent Project"
 __credits__ = ["Peter Maxwell", "Gavin Huttley", "Rob Knight",
                     "Matthew Wakefield", "Brett Easton"]
 __license__ = "GPL"
-__version__ = "1.5.1"
+__version__ = "1.5.3"
 __maintainer__ = "Gavin Huttley"
 __email__ = "gavin.huttley@anu.edu.au"
 __status__ = "Production"
@@ -376,6 +376,12 @@ motif  mprobs
         self.assertAlmostEquals(-250.686745262,
             likelihood_function.getLogLikelihood(),places=9)
     
+    def test_g_statistic(self):
+        likelihood_function = self._makeLikelihoodFunction()
+        self._setLengthsAndBetas(likelihood_function)
+        self.assertAlmostEquals(230.77670557,
+            likelihood_function.getGStatistic(),places=6)
+    
     def test_ancestralsequences(self):
         likelihood_function = self._makeLikelihoodFunction()
         self._setLengthsAndBetas(likelihood_function)
