@@ -4,18 +4,20 @@
 from cogent.util.unit_test import TestCase, main
 from cogent.maths.stats.test import tail, G_2_by_2,G_fit, likelihoods,\
     posteriors, bayes_updates, t_paired, t_one_sample, t_two_sample, \
-    t_one_observation,correlation, correlation_matrix, z_test, z_tailed_prob, \
-    t_tailed_prob, sign_test,\
-    reverse_tails, ZeroExpectedError, combinations, multiple_comparisons, \
-    multiple_inverse, multiple_n, fisher, regress, regress_major,\
-    f_value, f_two_sample, calc_contingency_expected, G_fit_from_Dict2D, \
-    chi_square_from_Dict2D, MonteCarloP, \
+    mc_t_two_sample, _permute_observations, t_one_observation, correlation, \
+    correlation_test, correlation_matrix, z_test, z_tailed_prob, \
+    t_tailed_prob, sign_test, reverse_tails, ZeroExpectedError, combinations, \
+    multiple_comparisons, multiple_inverse, multiple_n, fisher, regress, \
+    regress_major, f_value, f_two_sample, calc_contingency_expected, \
+    G_fit_from_Dict2D, chi_square_from_Dict2D, MonteCarloP, \
     regress_residuals, safe_sum_p_log_p, G_ind, regress_origin, stdev_from_mean, \
-    regress_R2, permute_2d, mantel, kendall_correlation, std, median,\
-    get_values_from_matrix, get_ltm_cells, distance_matrix_permutation_test,\
-    ANOVA_one_way, mw_test, mw_boot
+    regress_R2, permute_2d, mantel, mantel_test, _flatten_lower_triangle, \
+    pearson, spearman, _get_rank, kendall_correlation, std, median, \
+    get_values_from_matrix, get_ltm_cells, distance_matrix_permutation_test, \
+    ANOVA_one_way, mw_test, mw_boot, is_symmetric_and_hollow
 
-from numpy import array, reshape, arange, ones, testing, cov, sqrt
+from numpy import array, concatenate, fill_diagonal, reshape, arange, matrix, \
+        ones, testing, tril, cov, sqrt
 from cogent.util.dict2d import Dict2D
 import math
 from cogent.maths.stats.util import Numbers
@@ -23,13 +25,63 @@ from cogent.maths.stats.util import Numbers
 __author__ = "Rob Knight"
 __copyright__ = "Copyright 2007-2011, The Cogent Project"
 __credits__ = ["Rob Knight", "Catherine Lozupone", "Gavin Huttley",
-                    "Sandra Smit", "Daniel McDonald"]
+               "Sandra Smit", "Daniel McDonald", "Jai Ram Rideout",
+               "Michael Dwan"]
 __license__ = "GPL"
-__version__ = "1.5.1"
+__version__ = "1.5.3"
 __maintainer__ = "Rob Knight"
 __email__ = "rob@spot.colorado.edu"
 __status__ = "Production"
 
+class TestsHelper(TestCase):
+    """Class with utility methods useful for other tests."""
+
+    def setUp(self):
+        """Sets up variables used in the tests."""
+        # How many times a p-value should be tested to fall in a given range
+        # before failing the test.
+        self.p_val_tests = 10
+
+    def assertCorrectPValue(self, exp_min, exp_max, fn, args=None,
+                            kwargs=None, p_val_idx=0):
+        """Tests that the stochastic p-value falls in the specified range.
+
+        Performs the test self.p_val_tests times and fails if the observed
+        p-value does not fall into the specified range at least once. Each
+        p-value is also tested that it falls in the range 0.0 to 1.0.
+
+        This method assumes that fn is callable, and will unpack and pass args
+        and kwargs to fn if they are provided. It also assumes that fn returns
+        a single value (the p-value to be tested) or a tuple of results (any
+        length greater than or equal to 1), with the p-value at position
+        p_val_idx.
+
+        This is primarily used for testing the Mantel and correlation_test
+        functions.
+        """
+        found_match = False
+        for i in range(self.p_val_tests):
+            if args is not None and kwargs is not None:
+                obs = fn(*args, **kwargs)
+            elif args is not None:
+                obs = fn(*args)
+            elif kwargs is not None:
+                obs = fn(**kwargs)
+            else:
+                obs = fn()
+
+            try:
+                p_val = float(obs)
+            except TypeError:
+                p_val = obs[p_val_idx]
+
+            self.assertIsProb(p_val)
+            if p_val >= exp_min and p_val <= exp_max:
+                found_match = True
+                break
+        self.assertTrue(found_match)
+
+
 class TestsTests(TestCase):
     """Tests miscellaneous functions."""
     def test_std(self):
@@ -155,6 +207,13 @@ class TestsTests(TestCase):
         self.assertFloatEqual(regress(x, y), (-0.05322, 8.7038), 0.001)
         #higher precision from OpenOffice
         self.assertFloatEqual(regress(x, y), (-0.05322215,8.70402730))
+        
+        #add test to confirm no overflow error with large numbers
+        x = [32119,33831]
+        y = [2.28,2.43]
+        exp = (8.761682243E-05, -5.341209112E-01)
+        self.assertFloatEqual(regress(x,y),exp,0.001)
+ 
 
     def test_regress_origin(self):
         """regression slope constrained through origin should match Excel"""
@@ -162,6 +221,13 @@ class TestsTests(TestCase):
         y = array([4,2,6,8])
         self.assertFloatEqual(regress_origin(x, y), (1.9333333,0))
 
+        #add test to confirm no overflow error with large numbers
+        x = [32119,33831]
+        y = [2.28,2.43]
+        exp = (7.1428649481939822e-05, 0)
+        self.assertFloatEqual(regress_origin(x,y),exp,0.001)
+ 
+
     def test_regress_R2(self):
         """regress_R2 returns the R^2 value of a regression"""
         x = [1.0,2.0,3.0,4.0,5.0]
@@ -207,6 +273,15 @@ class TestsTests(TestCase):
             result = sign_test(success, trials, alt=alt)
             self.assertFloatEqual(result, p, eps=1e-5)
 
+    def test_permute_2d(self):
+        """permute_2d permutes rows and cols of a matrix."""
+        a = reshape(arange(9), (3,3))
+        self.assertEqual(permute_2d(a, [0,1,2]), a)
+        self.assertEqual(permute_2d(a, [2,1,0]), \
+            array([[8,7,6],[5,4,3],[2,1,0]]))
+        self.assertEqual(permute_2d(a, [1,2,0]), \
+            array([[4,5,3],[7,8,6],[1,2,0]]))
+
 
 class GTests(TestCase):
     """Tests implementation of the G tests for fit and independence."""
@@ -346,7 +421,7 @@ class GTests(TestCase):
         test3, csp3 = chi_square_from_Dict2D(matrix3)
         self.assertFloatEqual(test3, 7.6568405139833722)
         self.assertFloatEqual(csp3, 0.0217439383468)
-    
+
 
 class LikelihoodTests(TestCase):
     """Tests implementations of likelihood calculations."""
@@ -449,10 +524,12 @@ class BayesUpdateTests(TestCase):
             self.assertFloatEqualAbs(obs, exp, 1e-11)
         
 
-class StatTests(TestCase):
+class StatTests(TestsHelper):
     """Tests that the t and z tests are implemented correctly"""
 
     def setUp(self):
+        super(StatTests, self).setUp()
+
         self.x = [   
                 7.33, 7.49, 7.27, 7.93, 7.56,
                 7.81, 7.46, 6.94, 7.49, 7.44,
@@ -465,7 +542,6 @@ class StatTests(TestCase):
                 8.11, 7.66, 7.20, 7.25, 7.79,
             ]
 
-
     def test_t_paired_2tailed(self):
         """t_paired should match values from Sokal & Rohlf p 353"""
         x, y = self.x, self.y
@@ -549,7 +625,115 @@ class StatTests(TestCase):
         #book, because Sokal and Rohlf round their intermediate steps...
         self.assertFloatEqual(t_one_observation(x,sample),\
             (-1.5637254,0.1929248))
-    
+
+    def test_mc_t_two_sample(self):
+        """Test gives correct results with valid input data."""
+        # Verified against R's t.test() and perm.t.test().
+
+        # With numpy array as input.
+        exp = (-0.11858541225631833, 0.90756579317867436)
+        I =  array([7.2, 7.1, 9.1, 7.2, 7.3, 7.2, 7.5])
+        II = array([8.8, 7.5, 7.7, 7.6, 7.4, 6.7, 7.2])
+        obs = mc_t_two_sample(I, II)
+        self.assertFloatEqual(obs[:2], exp)
+        self.assertEqual(len(obs[2]), 999)
+        self.assertCorrectPValue(0.8, 0.9, mc_t_two_sample, [I, II],
+                                 p_val_idx=3)
+
+        # With python list as input.
+        exp = (-0.11858541225631833, 0.90756579317867436)
+        I =  [7.2, 7.1, 9.1, 7.2, 7.3, 7.2, 7.5]
+        II = [8.8, 7.5, 7.7, 7.6, 7.4, 6.7, 7.2]
+        obs = mc_t_two_sample(I, II)
+        self.assertFloatEqual(obs[:2], exp)
+        self.assertEqual(len(obs[2]), 999)
+        self.assertCorrectPValue(0.8, 0.9, mc_t_two_sample, [I, II],
+                                 p_val_idx=3)
+
+        exp = (-0.11858541225631833, 0.45378289658933718)
+        obs = mc_t_two_sample(I, II, tails='low')
+        self.assertFloatEqual(obs[:2], exp)
+        self.assertEqual(len(obs[2]), 999)
+        self.assertCorrectPValue(0.4, 0.47, mc_t_two_sample, [I, II],
+                {'tails':'low'}, p_val_idx=3)
+
+        exp = (-0.11858541225631833, 0.54621710341066287)
+        obs = mc_t_two_sample(I, II, tails='high', permutations=99)
+        self.assertFloatEqual(obs[:2], exp)
+        self.assertEqual(len(obs[2]), 99)
+        self.assertCorrectPValue(0.4, 0.62, mc_t_two_sample, [I, II],
+                {'tails':'high', 'permutations':99}, p_val_idx=3)
+
+        exp = (-2.8855783649036986, 0.99315596652421401)
+        obs = mc_t_two_sample(I, II, tails='high', permutations=99, exp_diff=1)
+        self.assertFloatEqual(obs[:2], exp)
+        self.assertEqual(len(obs[2]), 99)
+        self.assertCorrectPValue(0.55, 0.99, mc_t_two_sample, [I, II],
+                {'tails':'high', 'permutations':99, 'exp_diff':1}, p_val_idx=3)
+
+    def test_mc_t_two_sample_unbalanced_obs(self):
+        """Test gives correct results with unequal number of obs per sample."""
+        # Verified against R's t.test() and perm.t.test().
+        exp = (-0.10302479888889175, 0.91979753020527177)
+        I =  array([7.2, 7.1, 9.1, 7.2, 7.3, 7.2])
+        II = array([8.8, 7.5, 7.7, 7.6, 7.4, 6.7, 7.2])
+        obs = mc_t_two_sample(I, II)
+        self.assertFloatEqual(obs[:2], exp)
+        self.assertEqual(len(obs[2]), 999)
+        self.assertCorrectPValue(0.8, 0.9, mc_t_two_sample, [I, II],
+                                 p_val_idx=3)
+
+    def test_mc_t_two_sample_single_obs_sample(self):
+        """Test works correctly with one sample having a single observation."""
+        sample = array([4.02, 3.88, 3.34, 3.87, 3.18])
+        x = array([3.02])
+        exp = (-1.5637254,0.1929248)
+        obs = mc_t_two_sample(x, sample)
+        self.assertFloatEqual(obs[:2], exp)
+        self.assertFloatEqual(len(obs[2]), 999)
+        self.assertIsProb(obs[3])
+
+        obs = mc_t_two_sample(sample, x)
+        self.assertFloatEqual(obs[:2], exp)
+        self.assertFloatEqual(len(obs[2]), 999)
+        self.assertIsProb(obs[3])
+
+    def test_mc_t_two_sample_no_perms(self):
+        """Test gives empty permutation results if no perms are given."""
+        exp = (-0.11858541225631833, 0.90756579317867436, [], None)
+        I =  array([7.2, 7.1, 9.1, 7.2, 7.3, 7.2, 7.5])
+        II = array([8.8, 7.5, 7.7, 7.6, 7.4, 6.7, 7.2])
+        obs = mc_t_two_sample(I, II, permutations=0)
+        self.assertFloatEqual(obs, exp)
+
+    def test_mc_t_two_sample_no_mc(self):
+        """Test no MC stats if initial t-test is bad."""
+        x = array([1, 1, 1])
+        y = array([0, 0, 0])
+        self.assertEqual(mc_t_two_sample(x,x), (None, None, [], None))
+        self.assertEqual(mc_t_two_sample(x,y), (None, None, [], None))
+
+    def test_mc_t_two_sample_invalid_input(self):
+        """Test fails on various invalid input."""
+        self.assertRaises(ValueError, mc_t_two_sample, [1, 2, 3], [4., 5., 4.],
+                          tails='foo')
+        self.assertRaises(ValueError, mc_t_two_sample, [1, 2, 3], [4., 5., 4.],
+                          permutations=-1)
+        self.assertRaises(ValueError, mc_t_two_sample, [1], [4.])
+        self.assertRaises(ValueError, mc_t_two_sample, [1, 2], [])
+
+    def test_permute_observations(self):
+        """Test works correctly on small input dataset."""
+        I  = [10, 20., 1]
+        II = [2, 4, 5, 7]
+        obs = _permute_observations(I, II, 1)
+        self.assertEqual(len(obs[0]), 1)
+        self.assertEqual(len(obs[1]), 1)
+        self.assertEqual(len(obs[0][0]), len(I))
+        self.assertEqual(len(obs[1][0]), len(II))
+        self.assertFloatEqual(sorted(concatenate((obs[0][0], obs[1][0]))),
+                              sorted(I + II))
+
     def test_reverse_tails(self):
         """reverse_tails should return 'high' if tails was 'low' or vice versa"""
         self.assertEqual(reverse_tails('high'), 'low')
@@ -581,9 +765,248 @@ class StatTests(TestCase):
             (0.44721359549995793, 0.65472084601857694/2))
         self.assertFloatEqual(z_test(sample, 1, 10, 'low'), \
             (0.44721359549995793, 1-(0.65472084601857694/2)))
-    
-class CorrelationTests(TestCase):
-    """Tests of correlation coefficients."""
+
+
+class CorrelationTests(TestsHelper):
+    """Tests of correlation coefficients and Mantel test."""
+    def setUp(self):
+        """Sets up variables used in the tests."""
+        super(CorrelationTests, self).setUp()
+
+        # For testing spearman and correlation_test using method='spearman'.
+        # Taken from the Spearman wikipedia article. Also used for testing
+        # Pearson (verified with R).
+        self.data1 = [106, 86, 100, 101, 99, 103, 97, 113, 112, 110]
+        self.data2 = [7, 0, 27, 50, 28, 29, 20, 12, 6, 17]
+        
+        # For testing spearman.
+        self.a = [1,2,4,3,1,6,7,8,10,4]
+        self.b = [2,10,20,1,3,7,5,11,6,13]
+        self.c = [7,1,20,13,3,57,5,121,2,9]
+        self.r = (1.7,10,20,1.7,3,7,5,11,6.5,13)
+        self.x = (1, 2, 4, 3, 1, 6, 7, 8, 10, 4, 100, 2, 3, 77)
+
+        # Ranked copies for testing spearman.
+        self.b_ranked = [2, 7, 10, 1, 3, 6, 4, 8, 5, 9]
+        self.c_ranked = [5, 1, 8, 7, 3, 9, 4, 10, 2, 6]
+
+    def test_mantel(self):
+        """mantel should be significant for same matrix, not for random"""
+        a = reshape(arange(25), (5,5))
+        a = tril(a) + tril(a).T
+        fill_diagonal(a, 0)
+        b = a.copy()
+        #closely related -- should be significant
+        self.assertCorrectPValue(0.0, 0.049, mantel, (a, b, 1000))
+
+        c = reshape(ones(25), (5,5))
+        c[0, 1] = 3.0
+        c[1, 0] = 3.0
+        fill_diagonal(c, 0)
+        #not related -- should not be significant
+        self.assertCorrectPValue(0.06, 1.0, mantel, (a, c, 1000))
+
+    def test_mantel_test_one_sided_greater(self):
+        """Test one-sided mantel test (greater)."""
+        # This test output was verified by R (their mantel function does a
+        # one-sided greater test).
+        m1 = array([[0, 1, 2], [1, 0, 3], [2, 3, 0]])
+        m2 = array([[0, 2, 7], [2, 0, 6], [7, 6, 0]])
+        p, stat, perms = mantel_test(m1, m1, 999, alt='greater')
+        self.assertFloatEqual(stat, 1.0)
+        self.assertEqual(len(perms), 999)
+        self.assertCorrectPValue(0.09, 0.25, mantel_test, (m1, m1, 999),
+                                 {'alt':'greater'})
+
+        p, stat, perms = mantel_test(m1, m2, 999, alt='greater')
+        self.assertFloatEqual(stat, 0.755928946018)
+        self.assertEqual(len(perms), 999)
+        self.assertCorrectPValue(0.2, 0.5, mantel_test, (m1, m2, 999),
+                                 {'alt':'greater'})
+
+    def test_mantel_test_one_sided_less(self):
+        """Test one-sided mantel test (less)."""
+        # This test output was verified by R (their mantel function does a
+        # one-sided greater test, but I modified their output to do a one-sided
+        # less test).
+        m1 = array([[0, 1, 2], [1, 0, 3], [2, 3, 0]])
+        m2 = array([[0, 2, 7], [2, 0, 6], [7, 6, 0]])
+        m3 = array([[0, 0.5, 0.25], [0.5, 0, 0.1], [0.25, 0.1, 0]])
+        p, stat, perms = mantel_test(m1, m1, 999, alt='less')
+        self.assertFloatEqual(p, 1.0)
+        self.assertFloatEqual(stat, 1.0)
+        self.assertEqual(len(perms), 999)
+
+        p, stat, perms = mantel_test(m1, m2, 999, alt='less')
+        self.assertFloatEqual(stat, 0.755928946018)
+        self.assertEqual(len(perms), 999)
+        self.assertCorrectPValue(0.6, 1.0, mantel_test, (m1, m2, 999),
+                                 {'alt':'less'})
+
+        p, stat, perms = mantel_test(m1, m3, 999, alt='less')
+        self.assertFloatEqual(stat, -0.989743318611)
+        self.assertEqual(len(perms), 999)
+        self.assertCorrectPValue(0.1, 0.25, mantel_test, (m1, m3, 999),
+                                 {'alt':'less'})
+
+    def test_mantel_test_two_sided(self):
+        """Test two-sided mantel test."""
+        # This test output was verified by R (their mantel function does a
+        # one-sided greater test, but I modified their output to do a two-sided
+        # test).
+        m1 = array([[0, 1, 2], [1, 0, 3], [2, 3, 0]])
+        m2 = array([[0, 2, 7], [2, 0, 6], [7, 6, 0]])
+        m3 = array([[0, 0.5, 0.25], [0.5, 0, 0.1], [0.25, 0.1, 0]])
+        p, stat, perms = mantel_test(m1, m1, 999, alt='two sided')
+        self.assertFloatEqual(stat, 1.0)
+        self.assertEqual(len(perms), 999)
+        self.assertCorrectPValue(0.20, 0.45, mantel_test, (m1, m1, 999),
+                                 {'alt':'two sided'})
+
+        p, stat, perms = mantel_test(m1, m2, 999, alt='two sided')
+        self.assertFloatEqual(stat, 0.755928946018)
+        self.assertEqual(len(perms), 999)
+        self.assertCorrectPValue(0.6, 0.75, mantel_test, (m1, m2, 999),
+                                 {'alt':'two sided'})
+
+        p, stat, perms = mantel_test(m1, m3, 999, alt='two sided')
+        self.assertFloatEqual(stat, -0.989743318611)
+        self.assertEqual(len(perms), 999)
+        self.assertCorrectPValue(0.2, 0.45, mantel_test, (m1, m3, 999),
+                                 {'alt':'two sided'})
+
+    def test_mantel_test_invalid_distance_matrix(self):
+        """Test mantel test with invalid distance matrix."""
+        # Single asymmetric, non-hollow distance matrix.
+        self.assertRaises(ValueError, mantel_test, array([[1, 2], [3, 4]]),
+                          array([[0, 0], [0, 0]]), 999)
+
+        # Two asymmetric distance matrices.
+        self.assertRaises(ValueError, mantel_test, array([[0, 2], [3, 0]]),
+                          array([[0, 1], [0, 0]]), 999)
+
+    def test_mantel_test_invalid_input(self):
+        """Test mantel test with invalid input."""
+        self.assertRaises(ValueError, mantel_test, array([[1]]), array([[1]]),
+                          999, alt='foo')
+        self.assertRaises(ValueError, mantel_test, array([[1]]),
+            array([[1, 2], [3, 4]]), 999)
+        self.assertRaises(ValueError, mantel_test, array([[1]]),
+            array([[1]]), 0)
+        self.assertRaises(ValueError, mantel_test, array([[1]]),
+            array([[1]]), -1)
+
+    def test_is_symmetric_and_hollow(self):
+        """Should correctly test for symmetry and hollowness of dist mats."""
+        self.assertTrue(is_symmetric_and_hollow(array([[0, 1], [1, 0]])))
+        self.assertTrue(is_symmetric_and_hollow(matrix([[0, 1], [1, 0]])))
+        self.assertTrue(is_symmetric_and_hollow(matrix([[0.0, 0], [0.0, 0]])))
+        self.assertTrue(not is_symmetric_and_hollow(
+            array([[0.001, 1], [1, 0]])))
+        self.assertTrue(not is_symmetric_and_hollow(
+            array([[0, 1.1], [1, 0]])))
+        self.assertTrue(not is_symmetric_and_hollow(
+            array([[0.5, 1.1], [1, 0]])))
+
+    def test_flatten_lower_triangle(self):
+        """Test flattening various dms' lower triangulars."""
+        self.assertEqual(_flatten_lower_triangle(array([[8]])), [])
+        self.assertEqual(_flatten_lower_triangle(array([[1, 2], [3, 4]])), [3])
+        self.assertEqual(_flatten_lower_triangle(array([[1, 2, 3], [4, 5, 6],
+            [7, 8, 9]])), [4, 7, 8])
+
+    def test_pearson(self):
+        """Test pearson correlation method on valid data."""
+        # This test output was verified by R.
+        self.assertFloatEqual(pearson([1, 2], [1, 2]), 1.0)
+        self.assertFloatEqual(pearson([1, 2, 3], [1, 2, 3]), 1.0)
+        self.assertFloatEqual(pearson([1, 2, 3], [1, 2, 4]), 0.9819805)
+
+    def test_pearson_invalid_input(self):
+        """Test running pearson on bad input."""
+        self.assertRaises(ValueError, pearson, [1.4, 2.5], [5.6, 8.8, 9.0])
+        self.assertRaises(ValueError, pearson, [1.4], [5.6])
+
+    def test_spearman(self):
+        """Test the spearman function with valid input."""
+        # One vector has no ties.
+        exp = 0.3719581
+        obs = spearman(self.a, self.b)
+        self.assertFloatEqual(obs, exp)
+
+        # Both vectors have no ties.
+        exp = 0.2969697
+        obs = spearman(self.b, self.c)
+        self.assertFloatEqual(obs, exp)
+
+        # Both vectors have ties.
+        exp = 0.388381
+        obs = spearman(self.a, self.r)
+        self.assertFloatEqual(obs, exp)
+
+        exp = -0.17575757575757578
+        obs = spearman(self.data1, self.data2)
+        self.assertFloatEqual(obs, exp)
+
+    def test_spearman_no_variation(self):
+        """Test the spearman function with a vector having no variation."""
+        exp = 0.0
+        obs = spearman([1, 1, 1], [1, 2, 3])
+        self.assertFloatEqual(obs, exp)
+
+    def test_spearman_ranked(self):
+        """Test the spearman function with a vector that is already ranked."""
+        exp = 0.2969697
+        obs = spearman(self.b_ranked, self.c_ranked)
+        self.assertFloatEqual(obs, exp)
+
+    def test_spearman_one_obs(self):
+        """Test running spearman on a single observation."""
+        self.assertRaises(ValueError, spearman, [1.0], [5.0])
+
+    def test_spearman_invalid_input(self):
+        """Test the spearman function with invalid input."""
+        self.assertRaises(ValueError, spearman, [],[])
+        self.assertRaises(ValueError, spearman, self.a, [])
+        self.assertRaises(TypeError, spearman, {0:2}, [1,2,3])
+
+    def test_get_rank(self):
+        """Test the _get_rank function with valid input."""
+        exp = ([1.5,3.5,7.5,5.5,1.5,9.0,10.0,11.0,12.0,7.5,14.0,3.5,5.5,13.0],
+               4)
+        obs = _get_rank(self.x)
+        self.assertFloatEqual(exp,obs)
+
+        exp = ([1.5,3.0,5.5,4.0,1.5,7.0,8.0,9.0,10.0,5.5],2)
+        obs = _get_rank(self.a)
+        self.assertFloatEqual(exp,obs)
+
+        exp = ([2,7,10,1,3,6,4,8,5,9],0)
+        obs = _get_rank(self.b)
+        self.assertFloatEqual(exp,obs)
+
+        exp = ([1.5,7.0,10.0,1.5,3.0,6.0,4.0,8.0,5.0,9.0], 1)
+        obs = _get_rank(self.r)
+        self.assertFloatEqual(exp,obs)
+
+        exp = ([],0)
+        obs = _get_rank([])
+        self.assertEqual(exp,obs)
+
+    def test_get_rank_invalid_input(self):
+        """Test the _get_rank function with invalid input."""
+        vec = [1, 'a', 3, 2.5, 3, 1]
+        self.assertRaises(TypeError, _get_rank, vec)
+
+        vec = [1, 2, {1:2}, 2.5, 3, 1]
+        self.assertRaises(TypeError, _get_rank, vec)
+
+        vec = [1, 2, [23,1], 2.5, 3, 1]
+        self.assertRaises(TypeError, _get_rank, vec)
+
+        vec = [1, 2, (1,), 2.5, 3, 1]
+        self.assertRaises(TypeError, _get_rank, vec)
+
     def test_correlation(self):
         """Correlations and significance should match R's cor.test()"""
         x = [1,2,3,5]
@@ -603,6 +1026,134 @@ class CorrelationTests(TestCase):
         self.assertFloatEqualAbs(correlation(x,c), (0.3779645, 0.622), 1e-3)
         self.assertEqual(correlation(bad,bad), (1, 0))
 
+    def test_correlation_test_pearson(self):
+        """Test correlation_test using pearson on valid input."""
+        # These results were verified with R.
+
+        # Test with non-default confidence level and permutations.
+        obs = correlation_test(self.data1, self.data2, method='pearson',
+                               confidence_level=0.90, permutations=990)
+        self.assertFloatEqual(obs[:2], (-0.03760147, 0.91786297277172868))
+        self.assertEqual(len(obs[2]), 990)
+        for r in obs[2]:
+            self.assertTrue(r >= -1.0 and r <= 1.0)
+        self.assertCorrectPValue(0.9, 0.93, correlation_test,
+                (self.data1, self.data2),
+                {'method':'pearson', 'confidence_level':0.90,
+                 'permutations':990}, p_val_idx=3)
+        self.assertFloatEqual(obs[4], (-0.5779077, 0.5256224))
+
+        # Test with non-default tail type.
+        obs = correlation_test(self.data1, self.data2, method='pearson',
+                               confidence_level=0.90, permutations=990,
+                               tails='low')
+        self.assertFloatEqual(obs[:2], (-0.03760147, 0.45893148638586434))
+        self.assertEqual(len(obs[2]), 990)
+        for r in obs[2]:
+            self.assertTrue(r >= -1.0 and r <= 1.0)
+        self.assertCorrectPValue(0.41, 0.46, correlation_test,
+                (self.data1, self.data2),
+                {'method':'pearson', 'confidence_level':0.90,
+                    'permutations':990, 'tails':'low'}, p_val_idx=3)
+        self.assertFloatEqual(obs[4], (-0.5779077, 0.5256224))
+
+    def test_correlation_test_spearman(self):
+        """Test correlation_test using spearman on valid input."""
+        # This example taken from Wikipedia page:
+        # http://en.wikipedia.org/wiki/Spearman's_rank_correlation_coefficient
+        obs = correlation_test(self.data1, self.data2, method='spearman',
+                               tails='high')
+        self.assertFloatEqual(obs[:2], (-0.17575757575757578, 0.686405827612))
+        self.assertEqual(len(obs[2]), 999)
+        for rho in obs[2]:
+            self.assertTrue(rho >= -1.0 and rho <= 1.0)
+        self.assertCorrectPValue(0.67, 0.7, correlation_test,
+                (self.data1, self.data2),
+                {'method':'spearman', 'tails':'high'}, p_val_idx=3)
+        self.assertFloatEqual(obs[4],
+                              (-0.7251388558041697, 0.51034422964834503))
+
+        # The p-value is off because the example uses a one-tailed test, while
+        # we use a two-tailed test. Someone confirms the answer that we get
+        # here for a two-tailed test:
+        # http://stats.stackexchange.com/questions/22816/calculating-p-value-
+        #     for-spearmans-rank-correlation-coefficient-example-on-wikip
+        obs = correlation_test(self.data1, self.data2, method='spearman',
+                               tails=None)
+        self.assertFloatEqual(obs[:2],
+                (-0.17575757575757578, 0.62718834477648433))
+        self.assertEqual(len(obs[2]), 999)
+        for rho in obs[2]:
+            self.assertTrue(rho >= -1.0 and rho <= 1.0)
+        self.assertCorrectPValue(0.60, 0.64, correlation_test,
+                (self.data1, self.data2),
+                {'method':'spearman', 'tails':None}, p_val_idx=3)
+        self.assertFloatEqual(obs[4],
+                              (-0.7251388558041697, 0.51034422964834503))
+
+    def test_correlation_test_invalid_input(self):
+        """Test correlation_test using invalid input."""
+        self.assertRaises(ValueError, correlation_test, self.data1, self.data2,
+                          method='foo')
+        self.assertRaises(ValueError, correlation_test, self.data1, self.data2,
+                          tails='foo')
+        self.assertRaises(ValueError, correlation_test, self.data1, self.data2,
+                          permutations=-1)
+        self.assertRaises(ValueError, correlation_test, self.data1, self.data2,
+                          confidence_level=-1)
+        self.assertRaises(ValueError, correlation_test, self.data1, self.data2,
+                          confidence_level=1.1)
+        self.assertRaises(ValueError, correlation_test, self.data1, self.data2,
+                          confidence_level=0)
+        self.assertRaises(ValueError, correlation_test, self.data1, self.data2,
+                          confidence_level=0.0)
+        self.assertRaises(ValueError, correlation_test, self.data1, self.data2,
+                          confidence_level=1)
+        self.assertRaises(ValueError, correlation_test, self.data1, self.data2,
+                          confidence_level=1.0)
+
+    def test_correlation_test_no_permutations(self):
+        """Test correlation_test with no permutations."""
+        # These results were verified with R.
+        exp = (-0.2581988897471611, 0.7418011102528389, [], None,
+               (-0.97687328610475876, 0.93488023560400879))
+        obs = correlation_test([1, 2, 3, 4], [1, 2, 1, 1], permutations=0)
+        self.assertFloatEqual(obs, exp)
+
+    def test_correlation_test_perfect_correlation(self):
+        """Test correlation_test with perfectly-correlated input vectors."""
+        # These results were verified with R.
+        obs = correlation_test([1, 2, 3, 4], [1, 2, 3, 4])
+        self.assertFloatEqual(obs[:2],
+                (0.99999999999999978, 2.2204460492503131e-16))
+        self.assertEqual(len(obs[2]), 999)
+        for r in obs[2]:
+            self.assertTrue(r >= -1.0 and r <= 1.0)
+        self.assertCorrectPValue(0.06, 0.09, correlation_test,
+                ([1, 2, 3, 4], [1, 2, 3, 4]), p_val_idx=3)
+        self.assertFloatEqual(obs[4], (0.99999999999998879, 1.0))
+
+    def test_correlation_test_small_obs(self):
+        """Test correlation_test with a small number of observations."""
+        # These results were verified with R.
+        obs = correlation_test([1, 2, 3], [1, 2, 3])
+        self.assertFloatEqual(obs[:2], (1.0, 0))
+        self.assertEqual(len(obs[2]), 999)
+        for r in obs[2]:
+            self.assertTrue(r >= -1.0 and r <= 1.0)
+        self.assertCorrectPValue(0.3, 0.4, correlation_test,
+                ([1, 2, 3], [1, 2, 3]), p_val_idx=3)
+        self.assertFloatEqual(obs[4], (None, None))
+
+        obs = correlation_test([1, 2, 3], [1, 2, 3], method='spearman')
+        self.assertFloatEqual(obs[:2], (1.0, 0))
+        self.assertEqual(len(obs[2]), 999)
+        for r in obs[2]:
+            self.assertTrue(r >= -1.0 and r <= 1.0)
+        self.assertCorrectPValue(0.3, 0.4, correlation_test,
+                ([1, 2, 3], [1, 2, 3]), {'method':'spearman'}, p_val_idx=3)
+        self.assertFloatEqual(obs[4], (None, None))
+
     def test_correlation_matrix(self):
         """Correlations in matrix should match values from R"""
         a = [2,4,6,8]
@@ -717,28 +1268,6 @@ class Ftest(TestCase):
         p_val = MonteCarloP(val, random_vals, 'low')
         self.assertEqual(p_val, 0.4)
 
-    def test_permute_2d(self):
-        """permute_2d permutes rows and cols of a matrix."""
-        a = reshape(arange(9), (3,3))
-        self.assertEqual(permute_2d(a, [0,1,2]), a)
-        self.assertEqual(permute_2d(a, [2,1,0]), \
-            array([[8,7,6],[5,4,3],[2,1,0]]))
-        self.assertEqual(permute_2d(a, [1,2,0]), \
-            array([[4,5,3],[7,8,6],[1,2,0]]))
-
-    def test_mantel(self):
-        """mantel should be significant for same matrix, not for random"""
-        a = reshape(arange(25), (5,5))
-        b = a.copy()
-        b[-1,-1] = 26    #slight change
-        m = mantel(a, b, 1000)
-        #closely related -- should be significant
-        assert m < 0.05
-        c = reshape(ones(25), (5,5))
-        c[-1,-1] = 3
-        #not related -- should not be significant
-        m = mantel(a,c,1000)
-        assert m > 0.3, m
 
 class MannWhitneyTests(TestCase):
     """check accuracy of Mann-Whitney implementation"""
