@@ -25,16 +25,17 @@ from cogent.util.misc import (iterable, max_index, min_index,
     revComp, parse_command_line_parameters, safe_md5, 
     create_dir, handle_error_codes, identity, if_, deep_list, deep_tuple,
     combinate,gzip_dump,gzip_load,recursive_flatten_old,getNewId,toString,
-    timeLimitReached)
+    timeLimitReached, get_independent_coords, get_merged_by_value_coords,
+    get_merged_overlapping_coords, get_run_start_indices)
 from numpy import array
 from time import clock, sleep
 
 __author__ = "Rob Knight"
-__copyright__ = "Copyright 2007-2011, The Cogent Project"
+__copyright__ = "Copyright 2007-2012, The Cogent Project"
 __credits__ = ["Rob Knight", "Amanda Birmingham", "Sandra Smit",
                     "Zongzhi Liu", "Peter Maxwell", "Daniel McDonald"]
 __license__ = "GPL"
-__version__ = "1.5.1"
+__version__ = "1.5.3"
 __maintainer__ = "Rob Knight"
 __email__ = "rob@spot.colorado.edu"
 __status__ = "Production"
@@ -45,10 +46,12 @@ class UtilsTests(TestCase):
     def setUp(self):
         """ """
         self.files_to_remove = []
+        self.dirs_to_remove = []
         
     def tearDown(self):
         """ """
-        map(rmdir,self.files_to_remove)
+        map(remove,self.files_to_remove)
+        map(rmdir,self.dirs_to_remove)
 
     def test_identity(self):
         """should return same object"""
@@ -128,8 +131,15 @@ class UtilsTests(TestCase):
 
     def test_safe_md5(self):
         """Make sure we have the expected md5"""
-        exp = '8351caf492a97864cf8af555438ab851'
-        obs = safe_md5(open('test_util/__init__.py'))
+        exp = 'd3b07384d113edec49eaa6238ad5ff00'
+
+        tmp_fp = get_tmp_filename(prefix='test_safe_md5', suffix='txt')
+        self.files_to_remove.append(tmp_fp)
+        tmp_f = open(tmp_fp, 'w')
+        tmp_f.write('foo\n')
+        tmp_f.close()
+
+        obs = safe_md5(open(tmp_fp, 'U'))
         self.assertEqual(obs.hexdigest(),exp)
 
     def test_iterable(self):
@@ -221,9 +231,9 @@ class UtilsTests(TestCase):
         tmp_dir_path2 = get_random_directory_name(suppress_mkdir=True)
         tmp_dir_path3 = get_random_directory_name(suppress_mkdir=True)
 
-        self.files_to_remove.append(tmp_dir_path)
-        self.files_to_remove.append(tmp_dir_path2)
-        self.files_to_remove.append(tmp_dir_path3)
+        self.dirs_to_remove.append(tmp_dir_path)
+        self.dirs_to_remove.append(tmp_dir_path2)
+        self.dirs_to_remove.append(tmp_dir_path3)
 
         # create on existing dir raises OSError if fail_on_exist=True
         self.assertRaises(OSError, create_dir, tmp_dir_path,
@@ -637,6 +647,114 @@ class UtilsTests(TestCase):
          prefix='get_random_directory_test')
         self.assertTrue(exists(actual))
         rmdir(actual)
+    
+    def test_independent_spans(self):
+        """get_independent_coords returns truly non-overlapping (decorated) spans"""
+        # single span is returned
+        data = [(0, 20, 'a')]
+        got = get_independent_coords(data)
+        self.assertEqual(got, data)
+        
+        # multiple non-overlapping
+        data = [(20, 30, 'a'), (35, 40, 'b'), (65, 75, 'c')]
+        got = get_independent_coords(data)
+        self.assertEqual(got, data)
+        
+        # over-lapping first/second returns first occurrence by default
+        data = [(20, 30, 'a'), (25, 40, 'b'), (65, 75, 'c')]
+        got = get_independent_coords(data)
+        self.assertEqual(got, [(20, 30, 'a'), (65, 75, 'c')])
+        # but randomly the first or second if random_tie_breaker is chosen
+        got = get_independent_coords(data, random_tie_breaker=True)
+        self.assertTrue(got in ([(20, 30, 'a'), (65, 75, 'c')],
+                        [(25, 40, 'b'), (65, 75, 'c')]))
+        
+        # over-lapping second/last returns first occurrence by default
+        data = [(20, 30, 'a'), (30, 60, 'b'), (50, 75, 'c')]
+        got = get_independent_coords(data)
+        self.assertEqual(got, [(20, 30, 'a'), (30, 60, 'b')])
+        # but randomly the first or second if random_tie_breaker is chosen
+        got = get_independent_coords(data, random_tie_breaker=True)
+        self.assertTrue(got in ([(20, 30, 'a'), (50, 75, 'c')],
+                                [(20, 30, 'a'), (30, 60, 'b')]))
+        
+        # over-lapping middle returns first occurrence by default
+        data = [(20, 24, 'a'), (25, 40, 'b'), (30, 35, 'c'), (65, 75, 'd')]
+        got = get_independent_coords(data)
+        self.assertEqual(got, [(20, 24, 'a'), (25, 40, 'b'), (65, 75, 'd')])
+        
+        # but randomly the first or second if random_tie_breaker is chosen
+        got = get_independent_coords(data, random_tie_breaker=True)
+        self.assertTrue(got in ([(20, 24, 'a'), (25, 40, 'b'), (65, 75, 'd')],
+                                [(20, 24, 'a'), (30, 35, 'c'), (65, 75, 'd')]))
+    
+    def test_get_merged_spans(self):
+        """tests merger of overlapping spans"""
+        sample = [[0, 10], [12, 15], [13, 16], [18, 25], [19, 20]]
+        result = get_merged_overlapping_coords(sample)
+        expect = [[0, 10], [12, 16], [18, 25]]
+        self.assertEqual(result, expect)
+        sample = [[0, 10], [5, 9], [12, 16], [18, 20], [19, 25]]
+        result = get_merged_overlapping_coords(sample)
+        expect = [[0, 10], [12, 16], [18, 25]]
+        self.assertEqual(result, expect)
+    
+    def test_get_run_start_indices(self):
+        """return indices corresponding to start of a run of identical values"""
+        #       0  1  2  3  4  5  6  7
+        data = [1, 2, 3, 3, 3, 4, 4, 5]
+        expect = [[0, 1], [1, 2], [2, 3], [5, 4], [7, 5]]
+        got = get_run_start_indices(data)
+        self.assertEqual(list(got), expect)
+        
+        # raise an exception if try and provide a converter and num digits
+        def wrap_gen(): # need to wrap generator so we can actually test this
+            gen = get_run_start_indices(data, digits=1,
+                converter_func=lambda x: x)
+            def call():
+                for v in gen:
+                    pass
+            return call
+        
+        self.assertRaises(AssertionError, wrap_gen())
+    
+    def test_merged_by_value_spans(self):
+        """correctly merge adjacent spans with the same value"""
+        # initial values same
+        data = [[20, 21, 0], [21, 22, 0], [22, 23, 1], [23, 24, 0]]
+        self.assertEqual(get_merged_by_value_coords(data),
+                [[20, 22, 0], [22, 23, 1], [23, 24, 0]])
+        
+        # middle values same
+        data = [[20, 21, 0], [21, 22, 1], [22, 23, 1], [23, 24, 0]]
+        self.assertEqual(get_merged_by_value_coords(data),
+                [[20, 21, 0], [21, 23, 1], [23, 24, 0]])
+        
+        # last values same
+        data = [[20, 21, 0], [21, 22, 1], [22, 23, 0], [23, 24, 0]]
+        self.assertEqual(get_merged_by_value_coords(data),
+                [[20, 21, 0], [21, 22, 1], [22, 24, 0]])
+        
+        # all unique values
+        data = [[20, 21, 0], [21, 22, 1], [22, 23, 2], [23, 24, 0]]
+        self.assertEqual(get_merged_by_value_coords(data),
+                [[20, 21, 0], [21, 22, 1], [22, 23, 2], [23, 24, 0]])
+        
+        # all values same
+        data = [[20, 21, 0], [21, 22, 0], [22, 23, 0], [23, 24, 0]]
+        self.assertEqual(get_merged_by_value_coords(data),
+                [[20, 24, 0]])
+        
+        # all unique values to 2nd decimal
+        data = [[20, 21, 0.11], [21, 22, 0.12], [22, 23, 0.13], [23, 24, 0.14]]
+        self.assertEqual(get_merged_by_value_coords(data),
+                [[20, 21, 0.11], [21, 22, 0.12], [22, 23, 0.13], [23, 24, 0.14]])
+        
+        # all values same at 1st decimal
+        data = [[20, 21, 0.11], [21, 22, 0.12], [22, 23, 0.13], [23, 24, 0.14]]
+        self.assertEqual(get_merged_by_value_coords(data, digits=1),
+                [[20, 24, 0.1]])
+        
 
 
 class _my_dict(dict):
